/* Keyboard Layer/2
 *
 * Keyboard layout presentation map
 * Copyright (C) 2002 by Dmitry A.Steklenev
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR OR CONTRIBUTORS "AS IS"
 * AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * AUTHOR OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: layer_bundle.cpp,v 1.2 2002/04/01 18:31:28 glass Exp $
 */

#include "layer_bundle.h"
#include "layer_profiles.h"
#include "layer_engine.h"
#include "layer_ids.h"

#define  INCL_WIN
#include <os2.h>
#include <memory.h>
#include <string.h>
#include <stdio.h>

#define  ID_FLG(s) (( s[0] << 8 ) + s[1] )

/*--------------------------------------------------
 * Constructs keyboard layout presentation map
 *--------------------------------------------------*/
KLBundle::KLBundle( KLBroker* broker )
{
  lay_broker       = broker;
  lay_name[0]      = 0;
  lay_pref[0]      = 0;
  lay_cp           = 0;
  lay_multilayered = FALSE;
  lay_persistent   = TRUE;
  pri_bg_color     = 0;
  pri_fg_color     = 0;
  sec_bg_color     = 0;
  sec_fg_color     = 0;
  pri_icon         = 0;
  sec_icon         = 0;
}

/*--------------------------------------------------
 * Destructs keyboard layout presentation map
 *--------------------------------------------------*/
KLBundle::~KLBundle()
{
  if( pri_icon ) WinDestroyPointer( pri_icon );
  if( sec_icon ) WinDestroyPointer( sec_icon );

  save();
}

/*--------------------------------------------------
 * Saves keyboard layout presentation parameters
 *--------------------------------------------------*/
KLBundle& KLBundle::save()
{
  if( !lay_persistent )
  {
    properties parms;
    char key[64];

    strncpy( parms.name, lay_name, sizeof(parms.name));
    strncpy( parms.pref, lay_pref, sizeof(parms.pref));

    parms.pri_bg_color = pri_bg_color;
    parms.pri_fg_color = pri_fg_color;
    parms.sec_bg_color = sec_bg_color;
    parms.sec_fg_color = sec_fg_color;

    sprintf( key, "%06u@%s", lay_cp, lay_name );
    ini_put_data( key, &parms, sizeof(parms));

    lay_persistent = TRUE;
  }

  return *this;
}

/*--------------------------------------------------
 * Loads keyboard layout presentation parameters
 *--------------------------------------------------*/
KLBundle& KLBundle::load()
{
  if( stricmp( lay_name, lay_broker->selected()) == 0 &&
               lay_cp == lay_broker->cp_list()->selected )
  {
    return *this;
  }

  if( pri_icon ) if( !WinDestroyPointer( pri_icon )) DosBeep( 1000, 100 );
  if( sec_icon ) if( !WinDestroyPointer( sec_icon )) DosBeep( 1000, 100 );

  save   ();
  strncpy( lay_name, lay_broker->selected(), sizeof(lay_name));

  lay_cp           = lay_broker->cp_list()->selected;
  lay_multilayered = lay_broker->is_multilayered();
  pri_icon         = 0;
  sec_icon         = 0;
  lay_persistent   = TRUE;
  lay_pref[0]      = 0;

  properties parms;
  char key[64];

  if( lay_multilayered )
  {
    parms.pri_bg_color = 0x0000FFL;
    parms.pri_fg_color = 0xFFFFFFL;
    parms.sec_bg_color = 0xFF0000L;
    parms.sec_fg_color = 0xFFFFFFL;
  }
  else
  {
    parms.pri_bg_color = 0x000000L;
    parms.pri_fg_color = 0xFFFFFFL;
    parms.sec_bg_color = 0x000000L;
    parms.sec_fg_color = 0xFFFFFFL;
  }

  sprintf( key, "%06u@%s", lay_cp, lay_name );
  ini_get_data( key, &parms, sizeof(parms));

  pri_bg_color = parms.pri_bg_color;
  pri_fg_color = parms.pri_fg_color;
  sec_bg_color = parms.sec_bg_color;
  sec_fg_color = parms.sec_fg_color;

  if( lay_multilayered )
  {
    pri_icon = WinLoadPointer( HWND_DESKTOP, NULLHANDLE, ID_FLG_LATIN      );
    sec_icon = WinLoadPointer( HWND_DESKTOP, NULLHANDLE, ID_FLG( lay_name ));

    if( !sec_icon )
      sec_icon = WinLoadPointer( HWND_DESKTOP, NULLHANDLE, ID_FLG_NATIONAL );
  }
  else
  {
    pri_icon = WinLoadPointer( HWND_DESKTOP, NULLHANDLE, ID_FLG( lay_name ));
    sec_icon = WinLoadPointer( HWND_DESKTOP, NULLHANDLE, ID_FLG( lay_name ));

    if( !pri_icon )
      pri_icon = WinLoadPointer( HWND_DESKTOP, NULLHANDLE, ID_FLG_OTHER    );
    if( !sec_icon )
      sec_icon = WinLoadPointer( HWND_DESKTOP, NULLHANDLE, ID_FLG_OTHER    );
  }

  strncpy( lay_pref, parms.pref, sizeof(lay_pref));
  return *this;
}

/*--------------------------------------------------
 * Returns background color
 *--------------------------------------------------*/
LONG KLBundle::background()
{
  if( get_layer_id() == KL_LATIN )
    return load().pri_bg_color;
  else
    return load().sec_bg_color;
}

/*--------------------------------------------------
 * Returns foreground color
 *--------------------------------------------------*/
LONG KLBundle::foreground()
{
  if( get_layer_id() == KL_LATIN )
    return load().pri_fg_color;
  else
    return load().sec_fg_color;
}

/*--------------------------------------------------
 * Returns VDM screen border color
 *--------------------------------------------------*/
LONG KLBundle::vdm_border()
{
  return load().sec_bg_color;
}

/*--------------------------------------------------
 * Returns prefered keyboard layout name
 *--------------------------------------------------*/
const char* KLBundle::prefered()
{
  return load().lay_pref;
}

/*--------------------------------------------------
 * Returns layout caption
 *--------------------------------------------------*/
const char* KLBundle::caption()
{
  load();

  if( get_layer_id() == KL_LATIN && lay_multilayered )
    return "LATIN";
  else
    return lay_name;
}

/*--------------------------------------------------
 * Returns icon handle
 *--------------------------------------------------*/
HPOINTER KLBundle::icon()
{
  if( get_layer_id() == KL_LATIN )
    return load().pri_icon;
  else
    return load().sec_icon;
}

/*--------------------------------------------------
 * Sets background color
 *--------------------------------------------------*/
void KLBundle::set_background( LONG color )
{
  load();

  if( get_layer_id() == KL_LATIN )
    pri_bg_color = color;
  else
    sec_bg_color = color;

  lay_persistent = FALSE;
}

/*--------------------------------------------------
 * Sets foreground color
 *--------------------------------------------------*/
void KLBundle::set_foreground( LONG color )
{
  load();

  if( get_layer_id() == KL_LATIN )
    pri_fg_color = color;
  else
    sec_fg_color = color;

  lay_persistent = FALSE;
}

/*--------------------------------------------------
 * Sets prefered layout name
 *--------------------------------------------------*/
void KLBundle::set_prefered( const char* name )
{
  load();

  strncpy( lay_pref, name, sizeof(lay_pref));
  lay_persistent = FALSE;
}

