/* Keyboard Layer/2
 *
 * Keyboard layout bundle
 * Copyright (C) 2002 by Dmitry A.Steklenev
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR OR CONTRIBUTORS "AS IS"
 * AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * AUTHOR OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: layer_bundle.h,v 1.2 2002/04/01 18:31:29 glass Exp $
 */

/**@#-*/
#ifndef __LAYER_BUNDLE_H
#define __LAYER_BUNDLE_H

#include <os2def.h>
#include "layer_broker.h"
/**@#+*/

/**
 * Keyboard layouts presentation map.
 *
 * KLBundle is a collection of attributes that contain imaging
 * information for the current keyboard layout.
 *
 * @author Dmitry A Steklenev
 */

class KLBundle
{
  public:
    /** Constructs keyboard layout bundle. */
    KLBundle( KLBroker* broker );
    /** Destructs keyboard layout bundle. */
   ~KLBundle();

    /** Returns background color. */
    LONG background();
    /** Returns foreground color. */
    LONG foreground();
    /** Returns VDM screen border color. */
    LONG vdm_border();

    /** Returns icon handle. */
    HPOINTER icon();

    /** Sets background color. */
    void set_background( LONG color  );
    /** Sets foreground color. */
    void set_foreground( LONG color  );
    /** Sets prefered layout name. */
    void set_prefered  ( const char* );

    /** Returns prefered layout name. */
    const char* prefered();
    /** Returns keyboard layout caption. */
    const char* caption ();

  private:
    KLBundle& save();
    KLBundle& load();

    char lay_name[MAX_LNAME];
    char lay_pref[MAX_LNAME];

    KLBroker* lay_broker;
    ULONG     lay_cp;
    BOOL      lay_multilayered;
    BOOL      lay_persistent;
    LONG      pri_bg_color;
    LONG      pri_fg_color;
    LONG      sec_bg_color;
    LONG      sec_fg_color;
    HPOINTER  pri_icon;
    HPOINTER  sec_icon;

    #pragma pack(1)
    struct  properties
    {
       char name[MAX_LNAME];
       LONG pri_bg_color;
       LONG pri_fg_color;
       LONG sec_bg_color;
       LONG sec_fg_color;
       char pref[MAX_LNAME];
    };
    #pragma pack()
};

/**@#-*/
#endif
/**@#+*/
