/* Keyboard Layer/2
 *
 * Engine
 * Copyright (C) 1998 by Dmitry A.Steklenev
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR OR CONTRIBUTORS "AS IS"
 * AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * AUTHOR OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: layer_engine.cpp,v 1.2 2002/03/31 19:08:53 glass Exp $
 */

#include "layer_engine.h"
#include "layer_hotkey.h"
#include "layer_global.h"

#define  INCL_DOS
#define  INCL_WIN
#define  INCL_ERRORS
#include <os2.h>

/*--------------------------------------------------
 * Global variables
 *--------------------------------------------------*/

const char* _Export application_name = "Keyboard Layer/2";

HAB     hab                 = 0;  // Program anchor block
HWND    handle_of_indicator = 0;  // Layer window handle
HMODULE handle_of_dll       = 0;  // DLL handle
HWND    handle_of_lockup    = 0;  // Handle of lockup window
HVDD    handle_of_vdd       = 0;  // VDD handle
TID     tid_of_vdd          = 0;  // VDM thread identifier

/*--------------------------------------------------
 * Preferences
 *--------------------------------------------------*/

static ULONG  layer_id                 = KL_LATIN;
static ULONG  layer_default_id         = KL_NONE;
static BOOL   enable_vdm_synchronize   = FALSE;
static BOOL   enable_output_to_pipe    = FALSE;
static BOOL   enable_beeps             = FALSE;
static BOOL   enable_changer           = FALSE;
static BOOL   enable_transparent_focus = FALSE;
static BOOL   enable_float_on_top      = FALSE;
static CHRMSG changer_hotkeys          = {0,0,0,0,0};

/*--------------------------------------------------
 * Preferences control functions
 *--------------------------------------------------*/

void  EXPENTRY set_layer_id         ( LONG id     ) { layer_id = id;                     }
void  EXPENTRY set_vdm_synchronize  ( BOOL enable ) { enable_vdm_synchronize = enable;   }
void  EXPENTRY set_pipe_out         ( BOOL enable ) { enable_output_to_pipe = enable;    }
void  EXPENTRY set_changer          ( BOOL enable ) { enable_changer = enable;           }
void  EXPENTRY set_transparent_focus( BOOL enable ) { enable_transparent_focus = enable; }
void  EXPENTRY set_float_on_top     ( BOOL enable ) { enable_float_on_top = enable;      }

BOOL  EXPENTRY is_vdm_synchronized         () { return enable_vdm_synchronize;   }
BOOL  EXPENTRY is_beeps_enabled            () { return enable_beeps;             }
BOOL  EXPENTRY is_float_on_top_enabled     () { return enable_float_on_top;      }
BOOL  EXPENTRY is_vdm_supported            () { return handle_of_vdd;            }
BOOL  EXPENTRY is_changer_enabled          () { return enable_changer;           }
BOOL  EXPENTRY is_transparent_focus_enabled() { return enable_transparent_focus; }
BOOL  EXPENTRY is_pipe_out_enabled         () { return enable_output_to_pipe;    }

SHORT EXPENTRY get_layer_hotkeys   () { return KLHotKey::selected(); }
LONG  EXPENTRY get_layer_timeout   () { return KLHotKey::timeout (); }
LONG  EXPENTRY get_layer_id        () { return layer_id;             }
ULONG EXPENTRY get_default_layer_id() { return layer_default_id;     }

/*--------------------------------------------------
 * Enables/disables sound
 *--------------------------------------------------*/
void EXPENTRY set_beeps( BOOL enable )
{
  enable_beeps = enable;

  if( handle_of_vdd )
    DosRequestVDD( handle_of_vdd, 0, VLCMD_SETBEEP, sizeof(enable), &enable, 0, NULL );
}

/*--------------------------------------------------
 * Sets default layer id for new windows
 *--------------------------------------------------*/
void EXPENTRY set_default_layer_id( ULONG id )
{
  layer_default_id = id;

  if( handle_of_vdd )
    DosRequestVDD( handle_of_vdd, 0, VLCMD_SETFORNEW, sizeof(id), &id, 0, NULL );
}

/*--------------------------------------------------
 * Sets "Layer Change" hotkeys
 *--------------------------------------------------*/
void EXPENTRY set_layer_hotkeys( SHORT type )
{
  KLHotKey::select( type );

  if( handle_of_vdd )
    DosRequestVDD( handle_of_vdd, 0, VLCMD_SETSWITCH, sizeof(type), &type, 0, NULL );
}

/*--------------------------------------------------
 * Sets the current timeout value
 *--------------------------------------------------*/
void EXPENTRY set_layer_timeout( LONG timeout )
{
  KLHotKey::set_timeout( timeout );

  if( handle_of_vdd )
    DosRequestVDD( handle_of_vdd, 0, VLCMD_SETTMOUT, sizeof(timeout), &timeout, 0, NULL );
}

/*--------------------------------------------------
 * Sets the VDM screen border color
 *--------------------------------------------------*/
void EXPENTRY set_vdm_color( LONG rgb )
{
  if( handle_of_vdd )
  {
    if( !is_vdm_synchronized())
      rgb = 0xFF0040;

    DosRequestVDD( handle_of_vdd, 0, VLCMD_SETCOLOR, sizeof(rgb), &rgb, 0, NULL );
  }
}

/*--------------------------------------------------
 * Set layout changer hotkeys
 *--------------------------------------------------*/
void EXPENTRY set_changer_hotkeys( CHRMSG* key )
{
  changer_hotkeys = *key;
  changer_hotkeys.fs = changer_hotkeys.fs & (KC_ALT | KC_CTRL | KC_SHIFT );
}

/*--------------------------------------------------
 * Sound signal at turn on of the latin layout
 *--------------------------------------------------*/
void EXPENTRY beep_lat()
{
  if( is_beeps_enabled())
      DosBeep( BEEP_LAT, BEEP_LEN );
}

/*--------------------------------------------------
 * Sound signal at turn on of the national layout
 *--------------------------------------------------*/
void EXPENTRY beep_nat()
{
  if( is_beeps_enabled())
      DosBeep( BEEP_NAT, BEEP_LEN );
}

/*--------------------------------------------------
 * Checks "Layer Change" hotkeys
 *--------------------------------------------------*/
LONG is_hotkey( BYTE  scan , /* scan code           */
                LONG  id,    /* current layer state */
                ULONG time   /* press key time      */ )
{
  return KLHotKey::dispatch( scan, id, time );
}

/*--------------------------------------------------
 * Is specified window VDM
 *--------------------------------------------------*/
static BOOL is_vdm( HWND hwnd, PID *ppid )
{
  if( !hwnd ) return FALSE;

  SWCNTRL swcntrl;
  HSWITCH hswitch = WinQuerySwitchHandle( WinQueryWindow( hwnd, QW_PARENT ), 0 );

  if( hswitch )
  {
    if( WinQuerySwitchEntry( hswitch, &swcntrl ) == 0 )
    {
       if( ppid )
          *ppid = swcntrl.idProcess;
    }
    else
       hswitch = NULLHANDLE;
  }

  return hswitch && ( swcntrl.bProgType == PROG_VDM ||
                      swcntrl.bProgType == PROG_WINDOWEDVDM );
}

/*--------------------------------------------------
 * Get VDM layer state
 *--------------------------------------------------*/
static SHORT get_vdm_layer( PID pid )
{
  if( !handle_of_vdd )
    return KL_LATIN;

  SHORT state;
  DosRequestVDD( handle_of_vdd, pid, VLCMD_GETLAYER, 0, NULL, sizeof(state), &state );

  return state;
}

/*--------------------------------------------------
 * Sets layer for PM window
 *--------------------------------------------------*/
void EXPENTRY set_pm_layer( HWND hwnd, ULONG layer, ULONG flFlags )
{
  if( hwnd )
  {
    HENUM heEnum = WinBeginEnumWindows( hwnd );
    HWND  next;

    WinSetKbdLayer( hwnd, layer, flFlags );

    while(( next  = WinGetNextWindow( heEnum )) != NULLHANDLE )
      set_pm_layer( next, layer, flFlags );

    WinEndEnumWindows( heEnum );
  }
}

/*--------------------------------------------------
 * Keyboard Input Hook
 *--------------------------------------------------*/
BOOL EXPENTRY hook_char( HAB, PQMSG pqMsg, ULONG )
{
  if( get_layer_hotkeys() != XS_DONTME && pqMsg->msg == WM_CHAR )
  {
    // Check layout changer hotkeys

    if( is_changer_enabled() && !( CHARMSG(&(pqMsg->msg))->fs & KC_KEYUP )
        && CHARMSG(&(pqMsg->msg))->scancode )
    {
      CHRMSG keys = *CHARMSG(&(pqMsg->msg));
      keys.fs = keys.fs & (KC_ALT | KC_CTRL | KC_SHIFT );

      if( keys.fs == changer_hotkeys.fs &&
          keys.scancode == changer_hotkeys.scancode )
      {
        CHARMSG(&(pqMsg->msg))->fs      |= KC_VIRTUALKEY;
        CHARMSG(&(pqMsg->msg))->vkey     = 0;
        CHARMSG(&(pqMsg->msg))->scancode = 0;

        WinPostMsg( handle_of_indicator, WM_CHANGERPOPUP, 0, 0 );
      }
    }

    SHORT detected = KL_NONE;

    // Check scan code if key is up or press (not of auto repeat!)

    if( CHARMSG(&(pqMsg->msg))->fs & KC_SCANCODE &&
      ( CHARMSG(&(pqMsg->msg))->fs & KC_KEYUP    || !(CHARMSG(&(pqMsg->msg))->fs & KC_PREVDOWN )))
    {

        detected = is_hotkey( CHARMSG(&(pqMsg->msg))->scancode |
                            ( CHARMSG(&(pqMsg->msg))->fs & KC_KEYUP ? 0x80 : 0 ),
                              get_layer_id(), pqMsg->time );
    }

    // If this is a virtual key, reset to switch sequences

    if(!(CHARMSG(&(pqMsg->msg))->fs & KC_SCANCODE ))
        detected = is_hotkey( 0, get_layer_id(), 0 );

    if( detected != KL_NONE && !is_vdm(pqMsg->hwnd, NULL ))
    {
        HWND active = WinQueryActiveWindow( HWND_DESKTOP );

        if( active )
        {
          set_layer_id( detected );
          set_pm_layer( active, detected, 0 );

          if( detected == KL_LATIN )
            beep_lat();
          else
            beep_nat();

          WinPostMsg( handle_of_indicator, WM_LAYERCHANGED, 0, 0 );
        }
        return FALSE;
    }
  }

  if( pqMsg->msg == WM_CHAR
      &&  CHARMSG(&(pqMsg->msg))->fs & KC_VIRTUALKEY
      && !CHARMSG(&(pqMsg->msg))->vkey )
  {
    HWND focus = WinQueryFocus( HWND_DESKTOP );

    if( focus )
    {
      set_layer_id( WinQueryKbdLayer( focus));
      WinPostMsg  ( handle_of_indicator, WM_LAYERCHANGED, 0, 0 );
    }
  }

  return FALSE;
}

/*--------------------------------------------------
 * Lockup Hook
 *--------------------------------------------------*/
BOOL EXPENTRY hook_lockup( HAB hab, HWND hwnd )
{
  handle_of_lockup = hwnd;
  return TRUE;
}

/*--------------------------------------------------
 * Send Messages Hook
 *--------------------------------------------------*/
void EXPENTRY hook_send( HAB hab, PSMHSTRUCT psmh, BOOL )
{
  // Prepare new window

  if( psmh->msg == WM_CREATE || psmh->msg == WM_INITDLG )
  {
    HWND  owner   = WinQueryWindow( psmh->hwnd, QW_OWNER   );
    HWND  parent  = WinQueryWindow( psmh->hwnd, QW_PARENT  );
    HWND  desktop = WinQueryDesktopWindow( hab, NULLHANDLE );

    if( parent == desktop && (owner == desktop || owner == 0))
    {
      if( get_default_layer_id() != KL_NONE )
        WinSetWindowULong( psmh->hwnd, QWL_KBDLAYER, get_default_layer_id());
    }
    else
      WinSetWindowULong( psmh->hwnd, QWL_KBDLAYER,
                         owner  ? WinQueryKbdLayer( owner  ) : 0 |
                         parent ? WinQueryKbdLayer( parent ) : 0 );
  }

  // Catch lockup window destroying

  if( psmh->msg == WM_DESTROY && psmh->hwnd == handle_of_lockup )
    handle_of_lockup = 0;

  // Make layer window floating to top

  if( psmh->msg == WM_WINDOWPOSCHANGED
      && is_float_on_top_enabled()
      && !handle_of_lockup
    )
  {
      SWP   swpWin = *((PSWP)psmh->mp1);

      SWP   swpLay;
      RECTL recWin;
      RECTL recLay;
      RECTL recRes;

      WinQueryWindowPos( handle_of_indicator, &swpLay );

      recWin.xLeft   = swpWin.x;
      recWin.yBottom = swpWin.y;
      recWin.xRight  = swpWin.x + swpWin.cx;
      recWin.yTop    = swpWin.y + swpWin.cy;

      recLay.xLeft   = swpLay.x;
      recLay.yBottom = swpLay.y;
      recLay.xRight  = swpLay.x + swpLay.cx;
      recLay.yTop    = swpLay.y + swpLay.cy;

      if( WinIntersectRect( hab, &recRes, &recWin, &recLay ))
        WinPostMsg( handle_of_indicator, WM_FLOATTOTOP, MPFROMLONG((PSWP)(psmh->mp1)), 0 );

      return;
  }

  // Check layer state if window focus is changed

  if( psmh->msg == WM_FOCUSCHANGE && SHORT1FROMMP(psmh->mp1))
  {
    SHORT detected = get_layer_id();
    PID   pid_vdm;
    HWND  focus = WinQueryFocus( HWND_DESKTOP );

    if( focus )
    {
      if( is_vdm( focus, &pid_vdm ))
        detected = get_vdm_layer( pid_vdm );
      else
        detected = WinQueryKbdLayer( WinQueryFocus( HWND_DESKTOP ));
    }

    if( detected != get_layer_id())
    {
      set_layer_id( detected );
      WinPostMsg  ( handle_of_indicator, WM_LAYERCHANGED, 0, 0 );
    }
  }
}

/*--------------------------------------------------
 * VDM Events Handler
 *--------------------------------------------------*/
static void _System vdd_handler()
{
  VLEVENT event;

  for(;;)
  {
    if( DosRequestVDD( handle_of_vdd, 0, VLCMD_GETEVENT, 0, NULL, sizeof(event), &event ) != 0 )
      DosExit( EXIT_THREAD, 0 );

    if( event.EventType == VLEVN_LAYERCHANGED )
    {
      set_layer_id( event.EventData );
      WinPostMsg  ( handle_of_indicator, WM_LAYERCHANGED, 0, 0 );
    }
  }
}

/*--------------------------------------------------
 * Setup Hooks
 *--------------------------------------------------*/
void EXPENTRY engine_start( HAB indicator_hab,
                            HWND indicator_handle )
{
  hab = indicator_hab;
  handle_of_indicator = indicator_handle;

  if( !handle_of_dll )
    if( DosQueryModuleHandle( "LAYER.DLL", &handle_of_dll ) == NO_ERROR )
    {
      WinSetHook( hab, 0, HK_INPUT  , (PFN)hook_char  , handle_of_dll );
      WinSetHook( hab, 0, HK_SENDMSG, (PFN)hook_send  , handle_of_dll );
      WinSetHook( hab, 0, HK_LOCKUP , (PFN)hook_lockup, handle_of_dll );

      if( DosOpenVDD( "LAYER$", &handle_of_vdd ))
        handle_of_vdd = 0;
      else
      {
        DosCreateThread( &tid_of_vdd, (PFNTHREAD)&vdd_handler, 0, CREATE_READY, 12000L );

        BOOL enable = TRUE;
        DosRequestVDD( handle_of_vdd, 0, VLCMD_ENABLE, sizeof(enable), &enable, 0, NULL );
      }
    }

  if( get_default_layer_id() != KL_NONE )
    set_pm_layer( HWND_DESKTOP, get_default_layer_id(), 0 );
}

/*--------------------------------------------------
 * Release Hooks
 *--------------------------------------------------*/
void EXPENTRY engine_stop()
{
  if( handle_of_dll && hab )
  {
    WinReleaseHook( hab, 0, HK_INPUT  , (PFN)hook_char  , handle_of_dll );
    WinReleaseHook( hab, 0, HK_SENDMSG, (PFN)hook_send  , handle_of_dll );
    WinReleaseHook( hab, 0, HK_LOCKUP , (PFN)hook_lockup, handle_of_dll );

    // This function releases the input hook and broadcasts a WM_NULL message
    // to all top level windows so that they will release the DLL.  If we don't
    // do this, the DLL will remain locked and we'll have to reboot in order to
    // recompile.

    WinBroadcastMsg( HWND_DESKTOP, WM_NULL, 0, 0, BMSG_FRAMEONLY | BMSG_POST );
  }

  handle_of_dll = 0;

  if( handle_of_vdd )
  {
    BOOL enable = FALSE;
    DosRequestVDD( handle_of_vdd, 0, VLCMD_ENABLE, sizeof(enable), &enable, 0, NULL );
    DosCloseVDD  ( handle_of_vdd );
    handle_of_vdd = 0;
  }

  if( tid_of_vdd )
  {
    DosKillThread( tid_of_vdd );
    tid_of_vdd = 0;
  }
}

