/* Keyboard Layer/2
 *
 * "Layer Change" hotkeys management class
 * Copyright (C) 2001 Dmitry A.Steklenev
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR OR CONTRIBUTORS "AS IS"
 * AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * AUTHOR OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: layer_hotkey.cpp,v 1.3 2002/04/04 17:41:33 glass Exp $
 */

#define  INCL_WIN
#include <os2.h>

#include "layer_hotkey.h"
#include "layer_global.h"

#if defined(__cplusplus)
  KLHotKey::sequence KLHotKey::sw_latin;
  KLHotKey::sequence KLHotKey::sw_natio;
  KLHotKey::sequence KLHotKey::sw_current;
  ULONG              KLHotKey::sw_time;
  ULONG              KLHotKey::sw_timeout;
  int                KLHotKey::sw_hotkey;

  #define DECLARE(x) KLHotKey::##x
#else
  #pragma data_seg()

  static hkey_sequence sw_latin;
  static hkey_sequence sw_natio;
  static hkey_sequence sw_current;
  static ULONG         sw_time;
  static ULONG         sw_timeout;
  static int           sw_hotkey;

  #define DECLARE(x) hkey_##x
#endif

/*--------------------------------------------------
 * Sets the current timeout value
 *--------------------------------------------------*/
void DECLARE( set_timeout( ULONG time ))
{
  sw_timeout = time;
  sw_time    = 0;
}

/*--------------------------------------------------
 * Returns the current timeout value
 *--------------------------------------------------*/
ULONG DECLARE( timeout())
{
  return sw_timeout;
}

/*--------------------------------------------------
 * Returns the "Layer Change" hotkeys
 *--------------------------------------------------*/
int DECLARE( selected())
{
  return sw_hotkey;
}

/*--------------------------------------------------
 * Selects the "Layer Change" hotkeys
 *--------------------------------------------------*/
void DECLARE( select( int type ))
{
  sw_current.key_1st = 0;
  sw_current.key_2st = 0;
  sw_time   = 0;
  sw_hotkey = type;

  switch( type )
  {
    case XS_LCONTROL:
      sw_latin.key_1st = CS_LCONTROL;
      sw_latin.key_2st = CS_LCONTROL | CS_UP;
      sw_natio.key_1st = CS_LCONTROL;
      sw_natio.key_2st = CS_LCONTROL | CS_UP;
      break;

    case XS_LCTRANDS:
      sw_latin.key_1st = CS_LCONTROL;
      sw_latin.key_2st = CS_LSHIFT;
      sw_natio.key_1st = CS_LCONTROL;
      sw_natio.key_2st = CS_LSHIFT;
      break;

    case XS_LSHIFT:
      sw_latin.key_1st = CS_LSHIFT;
      sw_latin.key_2st = CS_LSHIFT | CS_UP;
      sw_natio.key_1st = CS_LSHIFT;
      sw_natio.key_2st = CS_LSHIFT | CS_UP;
      break;

    case XS_SHIFTS:
      sw_latin.key_1st = CS_LSHIFT;
      sw_latin.key_2st = CS_RSHIFT;
      sw_natio.key_1st = CS_LSHIFT;
      sw_natio.key_2st = CS_RSHIFT;
      break;

    case XS_LWIN:
      sw_latin.key_1st = CS_LWIN;
      sw_latin.key_2st = CS_LWIN | CS_UP;
      sw_natio.key_1st = CS_LWIN;
      sw_natio.key_2st = CS_LWIN | CS_UP;
      break;

    case XS_RCONTROL:
      sw_latin.key_1st = CS_RCONTROL;
      sw_latin.key_2st = CS_RCONTROL | CS_UP;
      sw_natio.key_1st = CS_RCONTROL;
      sw_natio.key_2st = CS_RCONTROL | CS_UP;
      break;

    case XS_RCTRLSH:
      sw_latin.key_1st = CS_RCONTROL;
      sw_latin.key_2st = CS_RCONTROL | CS_UP;
      sw_natio.key_1st = CS_RSHIFT;
      sw_natio.key_2st = CS_RSHIFT | CS_UP;
      break;

    case XS_RCTRANDS:
      sw_latin.key_1st = CS_RCONTROL;
      sw_latin.key_2st = CS_RSHIFT;
      sw_natio.key_1st = CS_RCONTROL;
      sw_natio.key_2st = CS_RSHIFT;

    case XS_RSHIFT:
      sw_latin.key_1st = CS_RSHIFT;
      sw_latin.key_2st = CS_RSHIFT | CS_UP;
      sw_natio.key_1st = CS_RSHIFT;
      sw_natio.key_2st = CS_RSHIFT | CS_UP;
      break;

    case XS_RWIN:
      sw_latin.key_1st = CS_RWIN;
      sw_latin.key_2st = CS_RWIN | CS_UP;
      sw_natio.key_1st = CS_RWIN;
      sw_natio.key_2st = CS_RWIN | CS_UP;
      break;

    case XS_ALTSH:
      sw_latin.key_1st = CS_LALT;
      sw_latin.key_2st = CS_LSHIFT;
      sw_natio.key_1st = CS_RALT;
      sw_natio.key_2st = CS_RSHIFT;
      break;

    case XS_WINMENU:
      sw_latin.key_1st = CS_WINMENU;
      sw_latin.key_2st = CS_WINMENU | CS_UP;
      sw_natio.key_1st = CS_WINMENU;
      sw_natio.key_2st = CS_WINMENU | CS_UP;
      break;

    default:
      sw_latin.key_1st = 0;
      sw_latin.key_2st = 0;
      sw_natio.key_1st = 0;
      sw_natio.key_2st = 0;
      sw_hotkey        = XS_DONTME;
  }
}

/*--------------------------------------------------
 * Dispatches the next pressed or released key
 *--------------------------------------------------*/
ULONG DECLARE( dispatch( BYTE scancode, ULONG current_layer, ULONG time ))
{
  BOOL is_natio;
  BOOL is_latin;
  BOOL is_timeout = sw_timeout > 0 && sw_time > 0 && (time - sw_time)/100 >= sw_timeout;

  sw_current.key_1st = sw_current.key_2st;
  sw_current.key_2st = scancode;
  sw_time            = time;

  is_natio = (sw_current.key_1st == sw_natio.key_1st &&
              sw_current.key_2st == sw_natio.key_2st );
  is_latin = (sw_current.key_1st == sw_latin.key_1st &&
              sw_current.key_2st == sw_latin.key_2st );

  if( !is_timeout )
  {
    if( is_latin && is_natio )
      return current_layer == KL_NATIONAL ? KL_LATIN : KL_NATIONAL;
    if( is_latin )
      return KL_LATIN;
    if( is_natio )
      return KL_NATIONAL;
  }

  return KL_NONE;
}
