/* Keyboard Layer/2
 *
 * "Layer Change" hotkeys management class
 * Copyright (C) 2001 Dmitry A.Steklenev
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR OR CONTRIBUTORS "AS IS"
 * AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * AUTHOR OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: layer_hotkey.h,v 1.1 2002/03/31 19:08:53 glass Exp $
 */

/**@#-*/
#ifndef  __LAYER_HOTKEY_H
#define  __LAYER_HOTKEY_H

#define  XS_LCONTROL 0x0000    // LEFT  CONTROL
#define  XS_LCTRANDS 0x0001    // LEFT  CONTROL AND SHIFT
#define  XS_LSHIFT   0x0002    // LEFT  SHIFT
#define  XS_SHIFTS   0x0003    // LEFT  SHIFT   AND RIGHT SHIFT
#define  XS_LWIN     0x0004    // LEFT  WINDOWS
#define  XS_RCONTROL 0x0005    // RIGHT CONTROL
#define  XS_RCTRLSH  0x0006    // RIGHT CONTROL OR  RIGHT SHIFT
#define  XS_RCTRANDS 0x0007    // RIGHT CONTROL AND SHIFT
#define  XS_RSHIFT   0x0008    // RIGHT SHIFT
#define  XS_RWIN     0x0009    // RIGHT WINDOWS
#define  XS_ALTSH    0x000A    // ALT AND SHIFTs
#define  XS_WINMENU  0x000B    // WINDOWS MENU
#define  XS_DONTME   0x000C    // Don't touch my keyboard

#define  CS_LCONTROL     29    // Left Control   0x1D
#define  CS_RCONTROL     91    // Right Control  0x5B
#define  CS_LSHIFT       42    // Left Shift     0x2A
#define  CS_RSHIFT       54    // Right Shift    0x36
#define  CS_LALT         56    // Left Alt       0x38
#define  CS_RALT         94    // Right Alt      0x5E
#define  CS_WINMENU     124    // Windows Menu   0x7C
#define  CS_LWIN        126    // Left Windows   0x7E
#define  CS_RWIN        127    // Right Windows  0x7F
#define  CS_UP          128    // Key released   0x80
/**@#+*/

#if defined(__cplusplus) || defined(__ccdoc__)

/**
 * "Layer Change" hotkeys management class.
 *
 * On keyboards used for bidirectional languages, two characters
 * (a Latin one and a "National Language" one) are engraved on each
 * key and the user can select which of these characters is used,
 * by selecting the active keyboard language (or "Keyboard Layer").
 * <p>
 * Changing the keyboard layer can be done by the user pressing
 * one of the "Layer Change" hotkeys.
 *
 * @author Dmitry A Steklenev
 */

class KLHotKey
{
  public:
    /** Selects the "Layer Change" hotkeys. */
    static void select  ( int type );
    /** Returns the "Layer Change" hotkeys. */
    static int  selected();

    /**
     * Dispatches the next pressed or released key.
     *
     * @return If the hotkey is pressed
     *         returns KL_LATIN or KL_NATIONAL. Otherwise returns KL_NONE.
     */

    static ULONG dispatch( BYTE scancode, ULONG current_layer, ULONG time );

    /** Sets the current timeout value. */
    static void  set_timeout( ULONG time );
    /** Returns the current timeout value. */
    static ULONG timeout();

  private:
    struct sequence {
      BYTE key_1st;
      BYTE key_2st;
    };

    static sequence sw_latin;
    static sequence sw_natio;
    static sequence sw_current;
    static ULONG    sw_time;
    static ULONG    sw_timeout;
    static int      sw_hotkey;
};

#else
    typedef struct {
      BYTE key_1st;
      BYTE key_2st;

    } hkey_sequence;

    #pragma alloc_text( CSWAP_TEXT, hkey_select      )
    #pragma alloc_text( CSWAP_TEXT, hkey_selected    )
    #pragma alloc_text( CSWAP_TEXT, hkey_dispatch    )
    #pragma alloc_text( CSWAP_TEXT, hkey_set_timeout )
    #pragma alloc_text( CSWAP_TEXT, hkey_timeout     )

    extern void  hkey_select     ( int type );
    extern int   hkey_selected   ( void );
    extern ULONG hkey_dispatch   ( BYTE scancode, ULONG current_layer, ULONG time );
    extern void  hkey_set_timeout( ULONG time );
    extern ULONG hkey_timeout    ( void );
#endif

/**@#-*/
#endif
