/* Keyboard Layer/2
 *
 * Main module
 * Copyright (C) 1998 by Dmitry A.Steklenev
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR OR CONTRIBUTORS "AS IS"
 * AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * AUTHOR OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: layer_main.cpp,v 1.8 2002/05/12 16:05:30 glass Exp $
 */

#include "layer_monitors.h"
#include "layer_profiles.h"
#include "layer_engine.h"
#include "layer_broker.h"
#include "layer_messages.h"
#include "layer_ids.h"
#include "layer_global.h"
#include "layer_bundle.h"

extern  const char* application_name;

#define  INCL_WIN
#define  INCL_GPI
#define  INCL_ERRORS
#define  INCL_DOS
#include <os2.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>

#define  __IMPLEMENTATION__
#include <trayapi.h>

HAB  hab                 = NULLHANDLE; // Program anchor block
HMQ  hmq                 = NULLHANDLE; // Message queue handle
HWND handle_of_indicator = NULLHANDLE; // Frame window handle
HWND handle_of_popup     = NULLHANDLE; // Pop-up menu handle
HWND handle_of_smenu     = NULLHANDLE; // SysTray pop-up menu handle
BOOL has_focus           = FALSE;      // Has focus
BOOL has_systray         = FALSE;      // SysTray supported

KLBroker broker;
KLBundle bundle( &broker );

/*--------------------------------------------------
 * Selects specifed keyboard layout
 *--------------------------------------------------*/
static void layout_select( const char* name )
{
  if( broker.select( name ) == NO_ERROR )
  {
    ini_put_chars( INI_LAYOUT, name   );
    set_vdm_color( bundle.vdm_border());
  }
  else
  {
    DosBeep( 1000, 100 );
  }

  WinPostMsg( handle_of_indicator, WM_LAYERCHANGED, 0, 0 );
}

/*--------------------------------------------------
 * Makes color index from RGB
 *--------------------------------------------------*/
static LONG color_index( HPS Hps, LONG RGBColor )
{
  // Create an entry in the color table for the given RGB value.
  // The new index is returned.  This allows a given RGB value
  // to be selected by index.

  LONG ClrTableInfo[4]; // Color table info

  // Get number of colors in current table
  if( GpiQueryColorData( Hps, 4L, ClrTableInfo ))

  // Create new index for this RGB color
  GpiCreateLogColorTable( Hps, 0L, LCOLF_CONSECRGB,
                          ClrTableInfo[QCD_LCT_HIINDEX] + 1,
                          1L,
                          &RGBColor );

  // Return index of new entry
  return ClrTableInfo[QCD_LCT_HIINDEX] + 1;
}

/*--------------------------------------------------
 * Querys window instance
 *--------------------------------------------------*/
static HWND query_window_instance( const char* pchClassWanted )
{
  HENUM heEnum;
  HWND  hwndTop;
  HWND  hwndClient;
  CHAR  achClass[256];

  heEnum = WinBeginEnumWindows( HWND_DESKTOP );

  hwndTop = WinGetNextWindow( heEnum );
  while( hwndTop != NULLHANDLE )
  {
    hwndClient = WinWindowFromID( hwndTop, FID_CLIENT );
    if( hwndClient != NULLHANDLE )
    {
      WinQueryClassName( hwndClient, sizeof(achClass), achClass );
      if( strcmp( achClass, pchClassWanted ) == 0 )
      {
        WinEndEnumWindows( heEnum );
        return hwndClient;
      }
    }
    hwndTop = WinGetNextWindow( heEnum );
  }

  WinEndEnumWindows( heEnum );
  return NULLHANDLE;
}

/*--------------------------------------------------
 * Loads pop-up menu
 *--------------------------------------------------*/
static HWND popup_load()
{
  handle_of_popup = WinLoadMenu( HWND_DESKTOP, NULLHANDLE, ID_MNU_MAIN );
  handle_of_smenu = WinLoadMenu( HWND_DESKTOP, NULLHANDLE, ID_MNU_TRAY );

  if( handle_of_popup )
  {
    MENUITEM           item;
    HWND               submenu_main;
    HWND               submenu_tray;
    KLBroker::kl_entry entry;
    ULONG              cp = broker.cp_list()->selected;

    WinSendMsg( handle_of_popup, MM_QUERYITEM,
                MPFROM2SHORT( ID_MNU_LAYOUT, TRUE ), MPFROMP(&item));

    submenu_main = item.hwndSubMenu;

    WinSendMsg( handle_of_smenu , MM_QUERYITEM,
                MPFROM2SHORT( ID_MNU_LAYOUT, TRUE ), MPFROMP(&item));

    submenu_tray = item.hwndSubMenu;

    memset( &item, 0, sizeof(item));

    item.iPosition = MIT_END;
    item.afStyle   = MIS_TEXT;

    if( broker.table_open() == NO_ERROR )
    {
       for( int i = 0; broker.next( &entry, cp ); i++ )
       {
         item.id = ID_LAY_FIRST + i;
         WinSendMsg( submenu_main, MM_INSERTITEM, MPFROMP(&item), MPFROMP(entry.name));
         WinSendMsg( submenu_tray, MM_INSERTITEM, MPFROMP(&item), MPFROMP(entry.name));
       }
       broker.table_close();
    }

  }
  return handle_of_popup;
}

/*--------------------------------------------------
 * Shows pop-up menu
 *--------------------------------------------------*/
static void popup_show( HWND handle, LONG x, LONG y )
{
  if( handle == handle_of_popup )
  {
    SET_ITEM( handle, ID_MNU_HIDE    , MSG_HIDE      );
    SET_ITEM( handle, ID_MNU_TASKLIST, MSG_TASKLIST  );
    SET_ITEM( handle, ID_MNU_TRANSPFC, MSG_TRANSPFC  );
    SET_ITEM( handle, ID_MNU_FLOAT   , MSG_FLOAT     );
  }
    SET_ITEM( handle, ID_MNU_SETUP   , MSG_SETUP     );
    SET_ITEM( handle, ID_MNU_CLOSE   , MSG_CLOSE     );
    SET_ITEM( handle, ID_MNU_LAYOUT  , MSG_SELLAYOUT );

  // Mark current keyboard layout in submenu

  MENUITEM item;

  WinSendMsg( handle, MM_QUERYITEM,
              MPFROM2SHORT( ID_MNU_LAYOUT, TRUE ), MPFROMP(&item));

  HWND  submenu    = item.hwndSubMenu;
  LONG  item_count = (LONG)WinSendMsg( handle_of_popup, MM_QUERYITEMCOUNT, 0, 0 );
  char  item_text[10];

  const char* current = broker.selected();

  for( int i = 0; i < item_count; i++ )
  {
    WinSendMsg( submenu, MM_QUERYITEMTEXT,
                MPFROM2SHORT( ID_LAY_FIRST+i, sizeof(item_text)),
                MPFROMP(&item_text));

    WinCheckMenuItem( submenu, ID_LAY_FIRST+i, stricmp( item_text, current ) == 0 );
  }

  WinPopupMenu( handle_of_indicator, handle_of_indicator,
                handle, x, y, ID_MNU_SETUP,
                PU_POSITIONONITEM | PU_HCONSTRAIN   | PU_VCONSTRAIN |
                PU_MOUSEBUTTON1   | PU_MOUSEBUTTON2 | PU_KEYBOARD   );
}

/*--------------------------------------------------
 * Returns selected from pop-up menu keyboard layout
 *--------------------------------------------------*/
static const char* popup_get_layout_name( ULONG item_id )
{
  MENUITEM item;
  static char name[MAX_LNAME];

  WinSendMsg( handle_of_popup, MM_QUERYITEM,
              MPFROM2SHORT( ID_MNU_LAYOUT, TRUE ), MPFROMP(&item));

  WinSendMsg( item.hwndSubMenu, MM_QUERYITEMTEXT,
              MPFROM2SHORT(item_id, sizeof(name)),
              MPFROMP(&name));
  return name;
}

/*--------------------------------------------------
 * Removes indicator from task list
 *--------------------------------------------------*/
static void indicator_remove_from_tasklist( BOOL remove )
{
  SWCNTRL swcntrl;
  HSWITCH hswitch = WinQuerySwitchHandle( handle_of_indicator, 0 );

  WinQuerySwitchEntry ( hswitch, &swcntrl );
  swcntrl.uchVisibility = remove ? SWL_INVISIBLE : SWL_VISIBLE;
  WinChangeSwitchEntry( hswitch, &swcntrl );

  WinCheckMenuItem ( handle_of_popup, ID_MNU_TASKLIST,  remove );
  WinEnableMenuItem( handle_of_popup, ID_MNU_HIDE    , !remove );
}

/*--------------------------------------------------
 * Sets transparent focus mode
 *--------------------------------------------------*/
static void indicator_set_transparent_focus( BOOL enable )
{
  set_transparent_focus( enable );
  WinCheckMenuItem( handle_of_popup, ID_MNU_TRANSPFC, enable );
}

/*--------------------------------------------------
 * Sets float on top mode
 *--------------------------------------------------*/
static void indicator_set_float_on_top( BOOL enable )
{
  set_float_on_top ( enable );
  WinCheckMenuItem ( handle_of_popup, ID_MNU_FLOAT   , enable );
  WinEnableMenuItem( handle_of_popup, ID_MNU_TRANSPFC, enable );
}

/*--------------------------------------------------
 * Repaints layer indicator
 *--------------------------------------------------*/
static void indicator_repaint( HWND hWnd )
{
  RECTL rec;
  SWP   swp;
  HPS   hps = WinGetPS( hWnd );

  LONG bg_color = bundle.background();
  LONG fg_color = bundle.foreground();

  WinQueryWindowPos( hWnd, &swp );

  rec.xLeft   = 0;
  rec.yBottom = 0;
  rec.xRight  = swp.cx;
  rec.yTop    = swp.cy;

  WinFillRect( hps, &rec, color_index( hps, bg_color ));
  WinDrawText( hps, -1L, (PCH)bundle.caption(), &rec,
               color_index( hps, fg_color ),
               color_index( hps, bg_color ),
               DT_CENTER | DT_VCENTER     );

  if( has_focus )
  {
    POINTL pos;

    GpiSetLineWidth( hps, LINEWIDTH_NORMAL   );
    GpiSetLineType ( hps, LINETYPE_ALTERNATE );
    GpiSetColor    ( hps, color_index( hps, fg_color ));

    pos.y = 1;
    pos.x = 1;
    GpiMove( hps, &pos );
    pos.x = swp.cx-2;
    pos.y = swp.cy-2;
    GpiBox( hps, DRO_OUTLINE, &pos, 0, 0 );
  }

  WinReleasePS( hps );
}

/*--------------------------------------------------
 * Resizes layer indicator
 *--------------------------------------------------*/
static void indicator_resize()
{
  HPS   hps  = WinGetPS( WinWindowFromID( handle_of_indicator, FID_CLIENT ));
  RECTL rect = { 0, 0, 32000, 32000 };

  WinDrawText( hps, -1L, " XXXXX ", &rect, 0L, 0L, DT_QUERYEXTENT | DT_LEFT );

  WinSetWindowPos( handle_of_indicator, 0, 0, 0,
                   rect.xRight - rect.xLeft   + 2,
                   rect.yTop   - rect.yBottom + 4, SWP_SIZE );

  WinReleasePS( hps );
}

/*--------------------------------------------------
 * Indicator presentation change
 *--------------------------------------------------*/
static void indicator_representation( HWND hWnd, LONG id )
{
  if( id == PP_FOREGROUNDCOLOR )
  {
    LONG fg_color = bundle.foreground();

    WinQueryPresParam( hWnd, PP_FOREGROUNDCOLOR, 0, 0, sizeof(fg_color), &fg_color, QPF_NOINHERIT );
    bundle.set_foreground( fg_color );
  }

  if( id == PP_BACKGROUNDCOLOR )
  {
    LONG bg_color = bundle.background();

    WinQueryPresParam( hWnd, PP_BACKGROUNDCOLOR, 0, 0, sizeof(bg_color), &bg_color, QPF_NOINHERIT );
    bundle.set_background( bg_color );
    set_vdm_color( bundle.vdm_border());
  }

  indicator_resize();
  WinInvalidateRect( hWnd, 0, TRUE );
  WinStoreWindowPos( application_name, INI_POS, handle_of_indicator );
}

/*--------------------------------------------------
 * Outputs current layout to pipe
 *--------------------------------------------------*/
void indicator_to_pipe()
{
  if( is_pipe_out_enabled())
  {
    FILE* pipe = fopen( "\\PIPE\\KBDLAY2", "wb" );

    if( pipe )
    {
      fprintf( pipe, "%s\n", bundle.caption());
      fflush ( pipe );
      fclose ( pipe );
    }
  }
}

/*--------------------------------------------------
 * Clears pipe
 *--------------------------------------------------*/
void indicator_clear_pipe()
{
  FILE* pipe = fopen( "\\PIPE\\KBDLAY2", "wb" );

  if( pipe )
  {
    fprintf( pipe, "\n" );
    fflush ( pipe );
    fclose ( pipe );
  }
}

/*--------------------------------------------------
 * Changes indicator icon
 *--------------------------------------------------*/
static void indicator_set_icon( HWND hWnd )
{
  if( has_systray )
  {
    WinSendMsg    ( handle_of_indicator, WM_SETICON, (MPARAM)bundle.icon(), 0 );
    ChangeTrayIcon( hWnd, 0 );
  }
}

/*--------------------------------------------------
 * Replays indicator focus
 *--------------------------------------------------*/
static void indicator_replay_focus( HWND hWnd, HWND hLoseFocus )
{
  if( hLoseFocus
      && hLoseFocus != handle_of_popup
      && !WinIsWindowShowing( hLoseFocus ))
  {
    HWND hNextFocus = WinQueryWindow( handle_of_indicator, QW_NEXTTOP );

    if( WinIsWindowShowing( hNextFocus ))
        WinSetWindowPos( hNextFocus, 0, 0, 0, 0, 0, SWP_ACTIVATE );
  }

  WinInvalidateRect( hWnd, 0, TRUE );
}

/*--------------------------------------------------
 * Creates changer
 *--------------------------------------------------*/
static void changer_create( HWND handle )
{
  KLBroker::kl_entry entry;

  const char* name = broker.selected();
  const char* pref = bundle.prefered();

  SET_TEXT( handle, DID_OK          , MSG_OK         );
  SET_TEXT( handle, DID_CANCEL      , MSG_CANCEL     );
  SET_TEXT( handle, ID_CHN_SELECTTXT, MSG_CHNGLAYOUT );

  if( broker.table_open() == NO_ERROR )
  {

    for( int i = 0; broker.next( &entry, broker.cp_list()->selected );)
    {
      if( stricmp( name, entry.name ) == 0 )
        continue;

      WinSendDlgItemMsg( handle, ID_CHN_LISTBOX, LM_INSERTITEM,
                                 MPFROMSHORT(LIT_END), MPFROMP(&entry.name));

      if( stricmp( pref, entry.name ) == 0 )
        WinSendDlgItemMsg( handle, ID_CHN_LISTBOX, LM_SELECTITEM,
                                   MPFROMSHORT(i), MPFROMSHORT(TRUE));
      ++i;
    }
    broker.table_close();
  }

  SWP swp;
  WinQueryWindowPos( handle, &swp );

  WinSetWindowPos( handle, 0,
                   (WinQuerySysValue( HWND_DESKTOP, SV_CXFULLSCREEN ) - swp.cx )/2,
                   (WinQuerySysValue( HWND_DESKTOP, SV_CYFULLSCREEN ) - swp.cy )/2,
                    0, 0, SWP_MOVE
                 );
}

/*--------------------------------------------------
 * Sets selected from changer keyboard layout
 *--------------------------------------------------*/
static void changer_select_layout( HWND handle )
{
  char  prefered[MAX_LNAME];
  ULONG selected = WinQueryLboxSelectedItem( WinWindowFromID( handle, ID_CHN_LISTBOX ));

  if( selected != LIT_NONE )
  {
    WinSendDlgItemMsg( handle, ID_CHN_LISTBOX, LM_QUERYITEMTEXT,
                               MPFROM2SHORT( selected, sizeof(prefered)),
                               MPFROMP(&prefered));
    if( strlen( prefered ))
    {
      bundle.set_prefered( prefered );
      layout_select( prefered );
    }
  }
}

/*--------------------------------------------------
 * Dispatch changer message
 *--------------------------------------------------*/
MRESULT EXPENTRY dispatch_changer_messages( HWND hWnd,
                                            ULONG msg, MPARAM mp1, MPARAM mp2 )
{
  switch( msg )
  {
    case WM_INITDLG:
      changer_create( hWnd );
      break;

    case WM_COMMAND:
      if( SHORT1FROMMP(mp1) == DID_OK )
          changer_select_layout( hWnd );
      break;

  }
  return WinDefDlgProc( hWnd, msg, mp1, mp2 );
}

/*--------------------------------------------------
 * Show popup layout changer
 *--------------------------------------------------*/
static void changer_show()
{
  WinDlgBox( HWND_DESKTOP, handle_of_indicator, dispatch_changer_messages,
                           NULLHANDLE, ID_WIN_CHANGER, NULL );
}

/*--------------------------------------------------
 * Indicator controls message processing
 *--------------------------------------------------*/
static void dispatch_indicator_controls( HWND hWnd, ULONG control_id )
{
  switch( control_id )
  {
    case ID_MNU_CLOSE:
      WinPostMsg( hWnd, WM_QUIT, 0, 0 );
      break;

    case ID_MNU_SETUP:
    {
      BOOL save_transparent = is_transparent_focus_enabled();

      if( has_focus )
        set_transparent_focus( FALSE );

      setup_preferences();
      set_transparent_focus( save_transparent );
      break;
    }

    case ID_MNU_HIDE:
      WinShowWindow( handle_of_indicator, FALSE );
      WinStoreWindowPos( application_name, INI_POS, handle_of_indicator );
      break;

    case ID_MNU_TRANSPFC:
      indicator_set_transparent_focus( !is_transparent_focus_enabled());
      ini_put_long( INI_TRANSPFC, is_transparent_focus_enabled());
      break;

    case ID_MNU_FLOAT:
      indicator_set_float_on_top( !is_float_on_top_enabled());
      ini_put_long( INI_FLOATONTOP, is_float_on_top_enabled());
      break;

    case ID_MNU_TASKLIST:
    {
      BOOL remove = !WinIsMenuItemChecked( handle_of_popup, ID_MNU_TASKLIST );

      indicator_remove_from_tasklist(remove);
      ini_put_long( INI_REMOVE, remove );
      break;
    }
  }

  // Change keyboard layout
  if( control_id >= ID_LAY_FIRST )
     layout_select( popup_get_layout_name(control_id));
}

/*--------------------------------------------------
 * Indicator window message processing
 *--------------------------------------------------*/
MRESULT EXPENTRY dispatch_indicator_messages( HWND hWnd,
                                              ULONG msg, MPARAM mp1, MPARAM mp2 )
{
  static USHORT mouse_drag_mode = FALSE;  // window in move mode
  static POINTL mouse_drag_start;         // start point

  switch( msg )
  {
    case WM_CREATE:
      InitializeTrayApi( hWnd );
      break;

    case WM_DDE_INITIATEACK:
    {
      DDEINIT* ddeinit = (DDEINIT*)mp2;

      if( strcmp( ddeinit->pszAppName, SZAPP   ) == 0 &&
          strcmp( ddeinit->pszTopic  , SZTOPIC ) == 0 )
      {
        has_systray = TRUE;
        AnswerTrayApiDdeAck( mp1 );
        AddTrayIcon( hWnd, 0 );
      }
      return MPFROMLONG( FALSE );
    }

    case WM_DESTROY:
      if( has_systray )
        DeleteTrayIcon( hWnd );
      break;

    case WM_ERASEBACKGROUND:
      indicator_repaint( hWnd );
      return MPFROMLONG( FALSE);

    case WM_PAINT:
      indicator_repaint( hWnd );
      break;

    case WM_PRESPARAMCHANGED:
      indicator_representation( hWnd, LONGFROMMP(mp1));
      break;

    case WM_LAYERCHANGED:
      indicator_repaint ( hWnd );
      indicator_set_icon( hWnd );
      indicator_to_pipe ();
      return MPFROMLONG ( TRUE );

    case WM_FLOATTOTOP:
      if( is_float_on_top_enabled())
      {
        set_float_on_top( FALSE);
        WinSetWindowPos ( handle_of_indicator, HWND_TOP, 0, 0, 0, 0, SWP_ZORDER );
        set_float_on_top( TRUE );
      }
      break;

    case WM_SETFOCUS:
      if((has_focus = LONGFROMMP(mp2))
          && is_float_on_top_enabled()
          && is_transparent_focus_enabled())
      {
        WinPostMsg( hWnd, WM_REPLAYFOCUS, mp1, mp2 );
      }
      else
        WinInvalidateRect( hWnd, 0, TRUE );

      break;

    case WM_REPLAYFOCUS:
      indicator_replay_focus( hWnd, LONGFROMMP(mp1));
      break;

    case WM_COMMAND:
      dispatch_indicator_controls( hWnd, SHORT1FROMMP(mp1));
      return MPFROMLONG( FALSE );

    case WM_BUTTON2CLICK | 0x2000:
    {
      POINTL ptl;
      WinQueryPointerPos( HWND_DESKTOP, &ptl );
      WinMapWindowPoints( HWND_DESKTOP, hWnd , &ptl, 1 );
      popup_show( handle_of_smenu, ptl.x, ptl.y );
      return MPFROMLONG( FALSE );
    }

    case WM_CONTEXTMENU:
      popup_show( handle_of_popup, SHORT1FROMMP(mp1), SHORT2FROMMP(mp1));
      return MPFROMLONG( FALSE );

    case WM_CHANGERPOPUP:
      set_changer( FALSE);
      changer_show();
      set_changer( TRUE );
      return MPFROMLONG ( FALSE);

    case WM_BUTTON1DOWN:
      mouse_drag_mode = TRUE;
      WinQueryPointerPos( HWND_DESKTOP, &mouse_drag_start );
      WinSetWindowPos( handle_of_indicator, HWND_TOP, 0,0,0,0, SWP_ZORDER );
      WinSetCapture( HWND_DESKTOP, hWnd );
      break;

    case WM_BUTTON1UP:
      mouse_drag_mode = FALSE;
      WinSetCapture( HWND_DESKTOP, NULLHANDLE );
      WinStoreWindowPos( application_name, INI_POS, handle_of_indicator );
      break;

    case WM_MOUSEMOVE:
      if( mouse_drag_mode )
      {
        SWP    swp;
        POINTL pos;
        WinQueryPointerPos( HWND_DESKTOP, &pos );

        if( mouse_drag_start.x != pos.x || mouse_drag_start.y != pos.y )
        {
          WinQueryWindowPos( handle_of_indicator, &swp );
          WinSetWindowPos  ( handle_of_indicator, HWND_TOP,
                             swp.x + pos.x - mouse_drag_start.x,
                             swp.y + pos.y - mouse_drag_start.y,
                             0, 0, SWP_MOVE );

          mouse_drag_start = pos;
        }
      }
      break;

  }
  return WinDefWindowProc( hWnd, msg, mp1, mp2 );
}

/*--------------------------------------------------
 * Terminate program
 *--------------------------------------------------*/
static void quit( int retcode )
{
  if( handle_of_popup     ) WinDestroyWindow  ( handle_of_popup     );
  if( handle_of_indicator ) WinDestroyWindow  ( handle_of_indicator );
  if( hmq                 ) WinDestroyMsgQueue( hmq );
  if( hab                 ) WinTerminate      ( hab );

  indicator_clear_pipe();
  exit( retcode );
}

/*--------------------------------------------------
 * Start here
 *--------------------------------------------------*/
void main( int argc, char **argv )
{
  if( !( hab = WinInitialize(0)) ||
      !( hmq = WinCreateMsgQueue( hab, 0 )) ||
      !( WinRegisterClass( hab, application_name, dispatch_indicator_messages, 0, 0 )) ||
      !( KLMsg::open_catalog( "layer.cat" ))
    )
  {
    quit( LAYER_EXIT_INITERROR );
  }

  if( argc == 2 && stricmp( argv[1], "/stop" ) == 0 )
  {
    HWND handle = query_window_instance( application_name );
    int  time   = 10;

    if( handle )
      WinPostMsg( handle, WM_QUIT, 0, 0 );

    while( handle && time-- && query_window_instance( application_name ))
      DosSleep( 1000 );

    quit( LAYER_EXIT_NORMAL );
  }

  if( query_window_instance( application_name ) != NULLHANDLE )
  {
    WinMessageBox( HWND_DESKTOP, HWND_DESKTOP, MSG_RUNONE,
                   application_name, 100, MB_OK | MB_ERROR | MB_MOVEABLE );

    quit( LAYER_EXIT_DUPLICATE );
  }

  // Restore saved keyboard layout

  char name[MAX_LNAME];
  ini_get_chars( INI_LAYOUT, name, sizeof(name), "" );

  if( strlen(name))
    broker.select(name);

  // Create indicator

  ULONG flCreateFlags = FCF_ICON | FCF_NOBYTEALIGN | FCF_TASKLIST | FCF_BORDER;

  handle_of_indicator = WinCreateStdWindow( HWND_DESKTOP,
                                            WS_VISIBLE | WS_SAVEBITS,
                                            &flCreateFlags,
                                            application_name,
                                            application_name,
                                            0, 0,
                                            ID_WIN_INDICATOR, 0 );
  if( !handle_of_indicator )
    quit( LAYER_EXIT_INITERROR );

  // Load pop-up menu

  if( !popup_load())
    quit( LAYER_EXIT_INITERROR );

  // Restore indicator state

  indicator_remove_from_tasklist ( ini_get_long( INI_REMOVE    , FALSE ));
  indicator_set_transparent_focus( ini_get_long( INI_TRANSPFC  , FALSE ));
  indicator_set_float_on_top     ( ini_get_long( INI_FLOATONTOP, TRUE  ));

  // Start engine and monitors

  engine_start     ( hab, handle_of_indicator );
  monitors_start   ();
  apply_preferences();

  // Restore indicator position

  if( !WinRestoreWindowPos( application_name, INI_POS, handle_of_indicator ))
  {
    SWP swp;
    indicator_resize ();
    WinQueryWindowPos( handle_of_indicator, &swp );

    WinSetWindowPos
    (
      handle_of_indicator,
      HWND_TOP,
      (WinQuerySysValue( HWND_DESKTOP, SV_CXSCREEN ) - swp.cx ),
      (WinQuerySysValue( HWND_DESKTOP, SV_CYSCREEN ) - swp.cy ),
      0, 0, SWP_MOVE
    );
  }
  else
  {
    indicator_resize();

    if( ini_get_long( INI_REMOVE, FALSE ) == TRUE )
      WinSetWindowPos ( handle_of_indicator, HWND_TOP, 0,0,0,0, SWP_SHOW );
  }

  // Message queue handling

  WinSendMsg( handle_of_indicator, WM_LAYERCHANGED, 0, 0 );

  QMSG qms;

  while( WinGetMsg( hab, &qms, 0, 0, 0 ))
     WinDispatchMsg( hab, &qms );

  engine_stop  ();
  monitors_stop();

  // Save window position and terminate application

  WinStoreWindowPos( application_name, INI_POS, handle_of_indicator );
  quit( LAYER_EXIT_NORMAL );
}

