/* Keyboard Layer/2
 *
 * Messages catalog management class
 * Copyright (C) 1998 by Dmitry A.Steklenev
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR OR CONTRIBUTORS "AS IS"
 * AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * AUTHOR OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: layer_messages.h,v 1.2 2002/03/31 19:08:53 glass Exp $
 */

/**@#-*/
#ifndef __LAYER_MESSAGES_H
#define __LAYER_MESSAGES_H

#include <nl_types.h>
/**@#+*/

/**
 * Messages catalog management class.
 * @author Dmitry A Steklenev
 */

class KLMsg {

  private:
     /** Destructs messages catalog. */
    ~KLMsg();

  public:
     /** Opens the messages catalog. */
     static int open_catalog( const char* catalog_file );
     /** Returns the message. */
     static char* get( int msg_no );
     /** Selects a country. */
     static void select_country( int country = 0 );

  private:
     static nl_catd msg_catd;
     static int     msg_country;
     static KLMsg   msg_sentry;
};

#if !defined(__ccdoc__)

#define MSG_SETUP        KLMsg::get(1 )
#define MSG_ABOUT        KLMsg::get(2 )
#define MSG_HIDE         KLMsg::get(3 )
#define MSG_TASKLIST     KLMsg::get(4 )
#define MSG_TRANSPFC     KLMsg::get(5 )
#define MSG_FLOAT        KLMsg::get(6 )
#define MSG_CLOSE        KLMsg::get(7 )
#define MSG_SWFRAME      KLMsg::get(8 )
#define MSG_SWRCONTROL   KLMsg::get(9 )
#define MSG_SWSHIFTS     KLMsg::get(10)
#define MSG_SWALTSHIFTS  KLMsg::get(11)
#define MSG_DONTTOUCH    KLMsg::get(12)
#define MSG_ENABLEPIPE   KLMsg::get(13)
#define MSG_BEEPAFTER    KLMsg::get(14)
#define MSG_VDMCOLOR     KLMsg::get(15)
#define MSG_OK           KLMsg::get(16)
#define MSG_DEFAULT      KLMsg::get(17)
#define MSG_CANCEL       KLMsg::get(18)
#define MSG_RUNONE       KLMsg::get(19)
#define MSG_TIMEOUT      KLMsg::get(20)
#define MSG_SWLALT       KLMsg::get(21)
#define MSG_SWRALT       KLMsg::get(22)
#define MSG_SWLCONTROL   KLMsg::get(23)
#define MSG_SWLSHIFT     KLMsg::get(24)
#define MSG_SWRSHIFT     KLMsg::get(25)
#define MSG_SWRCTRLSH    KLMsg::get(26)
#define MSG_LANGUAGE     KLMsg::get(27)
#define MSG_LNGSELECT    KLMsg::get(28)
#define MSG_LNGRUSSIAN   KLMsg::get(29)
#define MSG_LNGENGLISH   KLMsg::get(30)
#define MSG_LNGAUTO      KLMsg::get(31)
#define MSG_PAGEOF       KLMsg::get(32)
#define MSG_LAYDEFAULT   KLMsg::get(34)
#define MSG_LAYLATIN     KLMsg::get(35)
#define MSG_LAYNATIONAL  KLMsg::get(36)
#define MSG_LAYSYSTEM    KLMsg::get(37)
#define MSG_SELLAYOUT    KLMsg::get(38)
#define MSG_LAYCHANGER   KLMsg::get(39)
#define MSG_CHNGENABLE   KLMsg::get(40)
#define MSG_ASSIGN       KLMsg::get(41)
#define MSG_KEYPROMPT    KLMsg::get(42)
#define MSG_CHNGLAYOUT   KLMsg::get(43)
#define MSG_SWLWIN       KLMsg::get(44)
#define MSG_SWRWIN       KLMsg::get(45)
#define MSG_SWWINMENU    KLMsg::get(46)
#define MSG_SWLCTRANDS   KLMsg::get(47)
#define MSG_SWRCTRANDS   KLMsg::get(48)
#endif

/**@#-*/
#define SET_TEXT( hWnd, id, pMsg ) \
        WinSetDlgItemText( hWnd, id, pMsg )
#define SET_ITEM( hWnd, id, pMsg ) \
        WinSendMsg( hWnd, MM_SETITEMTEXT, MPFROMLONG(id), MPFROMP(pMsg))

#endif
/**@#+*/
