/* Keyboard Layer/2
 *
 * Fullscreen monitors support
 * Copyright (C) 1999 by Dmitry A.Steklenev
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR OR CONTRIBUTORS "AS IS"
 * AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * AUTHOR OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: layer_monitors.cpp,v 1.2 2002/03/31 19:08:53 glass Exp $
 */

#include "layer_monitors.h"
#include "layer_engine.h"
#include "layer_hotkey.h"
#include "layer_global.h"

#define  INCL_WIN
#define  INCL_DOS
#define  INCL_KBD
#define  INCL_ERRORS
#define  INCL_DOSDEVIOCTL
#include <os2.h>

/*--------------------------------------------------
 * Data
 *--------------------------------------------------*/

static HMONITOR handle_of_keyboard = 0; // Keyboard driver handle
static TID      monitor_tid[16];        // Monitors threads identificators
static HEV      monitor_started    = 0; // Notifying about end of monitor
                                        // initialization
static BOOL     extended_key = FALSE;

/*--------------------------------------------------
 * Translate scan code to ASCII
 *--------------------------------------------------*/
static void translate( KEYPACKET *KeyPack )
{
  static struct _DATAPACKET
  {
    UCHAR   XlatedChar;
    UCHAR   XlatedScan;
    UCHAR   DBCSStatus;
    UCHAR   DBCSShift;
    USHORT  ShiftState;
    ULONG   Milliseconds;
    USHORT  KbdDDFlagWord;
    USHORT  XlateFlags;
    USHORT  XlateState1;
    USHORT  XlateState2;

  } DataPacket = { 0,0,0,0,0,0,0,0,0,0 };

  struct _PARMPACKET
  {
    USHORT cp;
    USHORT reserved;

  } ParmPacket = { 0, 0 };

  DataPacket.XlatedChar    = KeyPack->XlatedChar;
  DataPacket.XlatedScan    = KeyPack->XlatedScan;
  DataPacket.DBCSStatus    = KeyPack->DBCSStatus;
  DataPacket.DBCSShift     = KeyPack->DBCSShift;
  DataPacket.ShiftState    = KeyPack->ShiftState;
  DataPacket.Milliseconds  = KeyPack->Milliseconds;
  DataPacket.KbdDDFlagWord = KeyPack->KbdDDFlagWord;
  DataPacket.XlateFlags    = 0;

  ULONG ParmLen = sizeof(ParmPacket);
  ULONG DataLen = sizeof(DataPacket);

  DosDevIOCtl( handle_of_keyboard, IOCTL_KEYBOARD, KBD_XLATESCAN,
                                   &ParmPacket, sizeof(ParmPacket), &ParmLen,
                                   &DataPacket, sizeof(DataPacket), &DataLen  );

  if( DataPacket.XlateFlags & 0x0001 && KeyPack->XlatedScan != 0x37 )
  {
    KeyPack->XlatedChar = DataPacket.XlatedChar;
    KeyPack->XlatedScan = DataPacket.XlatedScan;
    KeyPack->ShiftState = DataPacket.ShiftState;
  }
}

/*--------------------------------------------------
 * Set Layer
 *--------------------------------------------------*/
static void set_fs_layer( SHORT sw )
{
  struct _PARMPACKET
  {
    USHORT cp;
    USHORT layer;
  };

  _PARMPACKET ParmPacket = { 0, 0 };
  _PARMPACKET DataPacket = { 0, 0 };

  // Get current codepage

  ULONG ParmLen = sizeof(ParmPacket);
  ULONG DataLen = sizeof(DataPacket);

  DosDevIOCtl( handle_of_keyboard, IOCTL_KEYBOARD, KBD_GETCODEPAGEID,
                                   &ParmPacket, sizeof(ParmPacket), &ParmLen,
                                   &DataPacket, sizeof(DataPacket), &DataLen  );
  // Set layer

  ParmLen = sizeof(ParmPacket);
  DataLen = sizeof(DataPacket);

  set_layer_id( ParmPacket.layer = sw );
  DosDevIOCtl ( handle_of_keyboard, IOCTL_KEYBOARD, 0x58,
                                   &ParmPacket, sizeof(ParmPacket), &ParmLen,
                                   &DataPacket, sizeof(DataPacket), &DataLen  );
}

/*--------------------------------------------------
 * Monitor
 *--------------------------------------------------*/
static void _System monitor( ULONG id_session )
{
  MONBUF    monitor_inp_buf = { sizeof(MONBUF), 0 };
  MONBUF    monitor_out_buf = { sizeof(MONBUF), 0 };
  SHORT     session_layer   = KL_LATIN;
  KEYPACKET packet;
  SHORT     detected;
  BYTE      scan;

  DosSetPriority( PRTYS_THREAD,
                  PRTYC_TIMECRITICAL, PRTYD_MAXIMUM, 0 );

  // Register monitor

  if( DosMonReg(  handle_of_keyboard,
                  &monitor_inp_buf, &monitor_out_buf,
                  MONITOR_BEGIN, (USHORT)id_session ))
  {
    monitor_tid[id_session] = 0;
    DosPostEventSem( monitor_started );
    DosExit( EXIT_THREAD, 0 );
  }
  else
    DosPostEventSem( monitor_started );

  // Dispatch packet

  for(;;)
  {
    USHORT packet_len = sizeof(packet);
    DosMonRead((PBYTE)&monitor_inp_buf, 0, (PBYTE)&packet, &packet_len );

    if( get_layer_hotkeys() == XS_DONTME )
    {
      DosMonWrite( (PBYTE)&monitor_out_buf, (PBYTE)&packet, packet_len );
      continue;
    }

    /* debug

    printf( "paket MonFlagWord   : %02X\n"
            "      OriginScan    : %02X\n"
            "      XlatedChar    : %02X\n"
            "      XlatedScan    : %02X\n"
            "      DBCSStatus    : %02X\n"
            "      DBCSShift     : %02X\n"
            "      ShiftState    : %04X\n"
            "      Milliseconds  : %08X\n"
            "      KbdDDFlagWord : %04X\n",

            packet.MonFlagWord  ,
            packet.OriginScan   ,
            packet.XlatedChar   ,
            packet.XlatedScan   ,
            packet.DBCSStatus   ,
            packet.DBCSShift    ,
            packet.ShiftState   ,
            packet.Milliseconds ,
            packet.KbdDDFlagWord
          );
    */

    // Skip open, close and flush packets. Also, skip key was repeated
    // make of a toggle key

    if(( packet.MonFlagWord   & ( MF_FLUSH | MF_OPEN | MF_CLOSE )) ||
       ( packet.KbdDDFlagWord & KF_MULTIMAKE ))
    {
      DosMonWrite( (PBYTE)&monitor_out_buf, (PBYTE)&packet, packet_len );
      continue;
    }

    // Handle Multi-Layer NLS packet.
    // Layout index: 0,1,2 - latin, 3,4 - national
    // (from kbdbase.sys source):

    if(( packet.KbdDDFlagWord & KF_KEYTYPEMASK ) == KF_MLNOTIFICATION )
    {
      session_layer = packet.LayerIndex <= 2 ? KL_LATIN : KL_NATIONAL;
      DosMonWrite( (PBYTE)&monitor_out_buf, (PBYTE)&packet, packet_len );
      continue;
    }

    if( get_layer_id() != session_layer )
      set_fs_layer( session_layer );

    // Usual key

    if( !(packet.KbdDDFlagWord & KF_KEYTYPEMASK ) &&
        !(packet.DBCSStatus & SF_NOTCHAR))
    {
         translate( &packet );
    }

    // Checking switch sequence

    scan = packet.OriginScan;

    if( scan == 0xE0 )      // Extended scan code
      extended_key = TRUE;
    else
    {
      if( extended_key )    // Convert extended key to PM scan code
      {
        switch( scan & 0x7F )
        {
          case CS_LALT     : scan = CS_RALT;     break;
          case CS_LCONTROL : scan = CS_RCONTROL; break;
          case 0x5B        : scan = CS_LWIN;     break;
          case 0x5C        : scan = CS_RWIN;     break;
          case 0x5D        : scan = CS_WINMENU;  break;
        }

        scan |= ( packet.OriginScan & 0x80 );
        extended_key = FALSE;
      }

      if( scan )
        detected = is_hotkey( scan, session_layer, packet.Milliseconds );
      else
        detected = KL_NONE;

      if( detected != KL_NONE )
      {
        set_fs_layer( session_layer = detected     );

        if( session_layer == KL_LATIN )
            beep_lat();
        else
            beep_nat();
      }
    }

    // bscan = extract_scan_code( &packet, id_session );

    // Transfer packet

    DosMonWrite( (PBYTE)&monitor_out_buf, (PBYTE)&packet, packet_len );
  }
}

/*--------------------------------------------------
 * Start full screen sessions monitors
 *--------------------------------------------------*/
void monitors_start()
{
  // Open keyboard driver

  if( DosMonOpen( "KBD$", & handle_of_keyboard ))
    return;

  if( DosCreateEventSem( NULL, &monitor_started, 0, FALSE ))
    return;

  // Register monitors for all OS/2 fullscreen sessions

  for( ULONG id = 0; id <= 15; id++ )
  {
    ULONG pulPostCt;

    DosCreateThread ( monitor_tid +id, (PFNTHREAD)&monitor, id, CREATE_READY, 12000L );
    DosWaitEventSem ( monitor_started, (ULONG)SEM_INDEFINITE_WAIT );
    DosResetEventSem( monitor_started, &pulPostCt );
  }

  DosCloseEventSem( monitor_started );
}

/*--------------------------------------------------
 * Stop monitors
 *--------------------------------------------------*/
void monitors_stop()
{

  // Kill all threads with monitors

  for( ULONG id = 0; id <= 15; id++ )
  {
    if( monitor_tid[id] )
        DosKillThread( monitor_tid[id] );
  }

  // Close keyboard driver

  if(  handle_of_keyboard )
  {
     DosMonClose(  handle_of_keyboard );
     handle_of_keyboard = 0;
  }
}

