/* Keyboard Layer/2
 *
 * User profile management
 * Copyright (C) 1999 by Dmitry A.Steklenev
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR OR CONTRIBUTORS "AS IS"
 * AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * AUTHOR OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: layer_profiles.cpp,v 1.2 2002/03/31 19:08:53 glass Exp $
 */

#include "layer_profiles.h"

#define  INCL_WIN
#include <os2.h>
#include <string.h>

extern const char* application_name;

/*--------------------------------------------------
 * Puts data to user profile
 *--------------------------------------------------*/
void ini_put_data( const char* key,
                   const void* exp, ULONG len )
{
  PrfWriteProfileData( HINI_USERPROFILE,
                       application_name,
                       key, (PVOID)exp, len );
}

/*--------------------------------------------------
 * Retrieves data from user profile
 *--------------------------------------------------*/
ULONG ini_get_data( const char* key, void* exp, ULONG len )
{
  ULONG get_len = len;

  PrfQueryProfileData( HINI_USERPROFILE,
                       application_name,
                       key, exp, &get_len );
  return get_len;
}

/*--------------------------------------------------
 * Puts long value to user profile
 *--------------------------------------------------*/
void ini_put_long( const char *key, LONG exp )
{
  PrfWriteProfileData( HINI_USERPROFILE,
                       application_name,
                       key, &exp, sizeof(exp));
}

/*--------------------------------------------------
 * Retrieves long value from user profile
 *--------------------------------------------------*/
LONG ini_get_long( const char *key, LONG def )
{
  LONG  exp = def;
  ULONG len = sizeof(exp);

  return PrfQueryProfileData( HINI_USERPROFILE,
                              application_name,
                              key, &exp, &len ) ? exp : def;
}

/*--------------------------------------------------
 * Puts short value to user profile
 *--------------------------------------------------*/
void ini_put_short( const char *key, SHORT exp )
{
  PrfWriteProfileData( HINI_USERPROFILE,
                       application_name,
                       key, &exp, sizeof(exp));
}

/*--------------------------------------------------
 * Retrieves short value from user profile
 *--------------------------------------------------*/
SHORT ini_get_short( const char *key, SHORT def )
{
  SHORT exp = def;
  ULONG len = sizeof(exp);

  return PrfQueryProfileData( HINI_USERPROFILE,
                              application_name,
                              key, &exp, &len ) ? exp : def;
}

/*--------------------------------------------------
 * Puts characters string to user profile
 *--------------------------------------------------*/
void ini_put_chars( const char *key, const char* exp )
{
  PrfWriteProfileData( HINI_USERPROFILE,
                       application_name,
                       key, (PVOID)exp, strlen(exp)+1 );
}

/*--------------------------------------------------
 * Retrieves characters string from user profile
 *--------------------------------------------------*/
char* ini_get_chars( const char *key,
                     char* exp, ULONG len, char* def )
{
  ULONG get_len = len;

  if( !PrfQueryProfileData( HINI_USERPROFILE,
                            application_name, key, exp, &get_len ))
  {
    strcpy( exp, def );
  }

  return exp;
}

