/* Keyboard Layer/2
 *
 * User Preferences
 * Copyright (C) 1998 by Dmitry A.Steklenev
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR OR CONTRIBUTORS "AS IS"
 * AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * AUTHOR OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: layer_setup.cpp,v 1.2 2002/03/31 19:08:53 glass Exp $
 */

#include "layer_ids.h"
#include "layer_messages.h"
#include "layer_profiles.h"
#include "layer_hotkey.h"
#include "layer_engine.h"
#include "layer_global.h"
#include "layer_bundle.h"

#define  INCL_WIN
#include <os2.h>
#include <memory.h>
#include <stdio.h>
#include <stdlib.h>

MRESULT EXPENTRY setup_dispatch   ( HWND hWnd, ULONG msg, MPARAM mp1, MPARAM mp2 );
MRESULT EXPENTRY layouts_dispatch ( HWND hWnd, ULONG msg, MPARAM mp1, MPARAM mp2 );
MRESULT EXPENTRY language_dispatch( HWND hWnd, ULONG msg, MPARAM mp1, MPARAM mp2 );
MRESULT EXPENTRY about_dispatch   ( HWND hWnd, ULONG msg, MPARAM mp1, MPARAM mp2 );

HWND handle_of_notebook = NULLHANDLE;
HWND handle_of_setup    = NULLHANDLE;
HWND handle_of_language = NULLHANDLE;
HWND handle_of_about    = NULLHANDLE;
HWND handle_of_layouts  = NULLHANDLE;

const char* hotkey_string( const CHRMSG* );
void        hotkey_assign( HWND, CHRMSG* );

extern HWND handle_of_indicator;
extern KLBundle bundle;
CHRMSG changer_hotkeys;

/*--------------------------------------------------
 * Helpers
 *--------------------------------------------------*/

#define ADD_ITEM( handle, id, item    ) \
        WinSendDlgItemMsg( handle, id, LM_INSERTITEM, MPFROMSHORT(LIT_END), MPFROMP(item))
#define SEL_ITEM( handle, id, item_no ) \
        WinSendDlgItemMsg( handle, id, LM_SELECTITEM, MPFROMSHORT(item_no), MPFROMSHORT(TRUE))
#define GET_ITEM( handle, id          ) \
        WinQueryLboxSelectedItem( WinWindowFromID( handle, id ))

#define SET_CHECKBOX( handle, id, value ) \
        WinSendDlgItemMsg( handle, id, BM_SETCHECK, MPFROMLONG(value), 0 )
#define GET_CHECKBOX( handle, id        ) \
        SHORT1FROMMR( WinSendDlgItemMsg( handle, id, BM_QUERYCHECK, 0, 0 ))

#define SET_RADIOBOX( handle, id, value ) \
        WinSendDlgItemMsg( handle, id, BM_SETCHECK, MPFROMLONG(value), 0 )
#define GET_RADIOBOX( handle, id        ) \
        SHORT1FROMMR( WinSendDlgItemMsg( handle, id, BM_QUERYCHECKINDEX, 0, 0 ))

#define SET_SPIN( handle, id, value  ) \
        WinSendDlgItemMsg( handle, id, SPBM_SETCURRENTVALUE, MPFROMLONG(value), 0 )
#define GET_SPIN( handle, id, value  ) \
        WinSendDlgItemMsg( handle, id, SPBM_QUERYVALUE, MPFROMP(value), 0 )

#define ENABLEID( handle, id, enable ) \
        WinEnableWindow( WinWindowFromID( handle, id ), enable );

/*--------------------------------------------------
 * Create setup dialog
 *--------------------------------------------------*/
static void setup_create( HWND dialog_handle )
{
  SET_TEXT( dialog_handle, ID_SET_SWFRAME, MSG_SWFRAME     );
  SET_TEXT( dialog_handle, ID_SET_BEEP   , MSG_BEEPAFTER   );
  SET_TEXT( dialog_handle, ID_SET_PIPES  , MSG_ENABLEPIPE  );
  SET_TEXT( dialog_handle, ID_SET_COSYN  , MSG_VDMCOLOR    );
  SET_TEXT( dialog_handle, ID_SET_TIMETXT, MSG_TIMEOUT     );

  ADD_ITEM( dialog_handle, ID_SET_SWITCH , MSG_SWLCONTROL  );
  ADD_ITEM( dialog_handle, ID_SET_SWITCH , MSG_SWLCTRANDS  );
  ADD_ITEM( dialog_handle, ID_SET_SWITCH , MSG_SWLSHIFT    );
  ADD_ITEM( dialog_handle, ID_SET_SWITCH , MSG_SWSHIFTS    );
  ADD_ITEM( dialog_handle, ID_SET_SWITCH , MSG_SWLWIN      );

  ADD_ITEM( dialog_handle, ID_SET_SWITCH , MSG_SWRCONTROL  );
  ADD_ITEM( dialog_handle, ID_SET_SWITCH , MSG_SWRCTRLSH   );
  ADD_ITEM( dialog_handle, ID_SET_SWITCH , MSG_SWRCTRANDS  );
  ADD_ITEM( dialog_handle, ID_SET_SWITCH , MSG_SWRSHIFT    );
  ADD_ITEM( dialog_handle, ID_SET_SWITCH , MSG_SWRWIN      );

  ADD_ITEM( dialog_handle, ID_SET_SWITCH , MSG_SWALTSHIFTS );
  ADD_ITEM( dialog_handle, ID_SET_SWITCH , MSG_SWWINMENU   );

  ADD_ITEM( dialog_handle, ID_SET_SWITCH , MSG_DONTTOUCH   );

  SEL_ITEM( dialog_handle, ID_SET_SWITCH , ini_get_short( INI_SWITCH, XS_RCONTROL ));

  SET_CHECKBOX( dialog_handle, ID_SET_BEEP   , ini_get_long( INI_BEEP   , FALSE ));
  SET_CHECKBOX( dialog_handle, ID_SET_PIPES  , ini_get_long( INI_PIPES  , FALSE ));
  SET_CHECKBOX( dialog_handle, ID_SET_COSYN  , ini_get_long( INI_COSYN  , FALSE ));
  SET_SPIN    ( dialog_handle, ID_SET_TIMEOUT, ini_get_long( INI_TIMEOUT, 0     ));

  ENABLEID( dialog_handle, ID_SET_COSYN, is_vdm_supported());
}

/*--------------------------------------------------
 * Set setup values to default
 *--------------------------------------------------*/
static void setup_default( HWND dialog_handle )
{
  SEL_ITEM    ( dialog_handle, ID_SET_SWITCH , XS_RCONTROL );
  SET_CHECKBOX( dialog_handle, ID_SET_BEEP   , FALSE       );
  SET_CHECKBOX( dialog_handle, ID_SET_PIPES  , FALSE       );
  SET_CHECKBOX( dialog_handle, ID_SET_COSYN  , FALSE       );
  SET_SPIN    ( dialog_handle, ID_SET_TIMEOUT, 0           );
}

/*--------------------------------------------------
 * Save setup preferences
 *--------------------------------------------------*/
static void setup_save()
{
  LONG timeout;

  GET_SPIN( handle_of_setup, ID_SET_TIMEOUT, &timeout );

  ini_put_short( INI_SWITCH    , GET_ITEM    ( handle_of_setup, ID_SET_SWITCH ));
  ini_put_long ( INI_BEEP      , GET_CHECKBOX( handle_of_setup, ID_SET_BEEP   ));
  ini_put_long ( INI_PIPES     , GET_CHECKBOX( handle_of_setup, ID_SET_PIPES  ));
  ini_put_long ( INI_COSYN     , GET_CHECKBOX( handle_of_setup, ID_SET_COSYN  ));
  ini_put_long ( INI_TIMEOUT   , timeout );
}

/*--------------------------------------------------
 * Apply setup preferences
 *--------------------------------------------------*/
static void setup_apply()
{
  set_layer_hotkeys  ( ini_get_short( INI_SWITCH    , XS_RCONTROL  ));
  set_layer_timeout  ( ini_get_long ( INI_TIMEOUT   , 0            ));
  set_beeps          ( ini_get_long ( INI_BEEP      , FALSE        ));
  set_pipe_out       ( ini_get_long ( INI_PIPES     , FALSE        ));
  set_vdm_synchronize( ini_get_long ( INI_COSYN     , FALSE        ));

  if( !is_pipe_out_enabled())
    indicator_clear_pipe();

  set_vdm_color( bundle.vdm_border());
}

/*--------------------------------------------------
 * Setup dialog message processing
 *--------------------------------------------------*/
MRESULT EXPENTRY setup_dispatch( HWND hWnd, ULONG msg, MPARAM mp1, MPARAM mp2 )
{
  switch ( msg )
  {
    case WM_INITDLG:
      setup_create( hWnd );
      break;

    case WM_SETDEFAULT:
      setup_default( hWnd );
      return MRFROMLONG( TRUE );

    case WM_CONTROL:
      if( SHORT2FROMMP(mp1) == CBN_EFCHANGE && SHORT1FROMMP(mp1) == ID_SET_SWITCH )
      {
        BOOL enable_switch = GET_ITEM( hWnd, ID_SET_SWITCH ) != XS_DONTME;

        ENABLEID( hWnd, ID_SET_BEEP   , enable_switch );
        ENABLEID( hWnd, ID_SET_TIMEOUT, enable_switch );
        ENABLEID( hWnd, ID_SET_TIMETXT, enable_switch );
      }
      break;

    case WM_COMMAND:
      if( SHORT1FROMMP(mp1) == DID_CANCEL )
        return MRFROMLONG( TRUE );

      break;
  }
  return WinDefDlgProc( hWnd, msg, mp1, mp2 );
}

/*---------------------------------------------------
 * Create layouts dialog
 *--------------------------------------------------*/
static void layouts_create( HWND dialog_handle )
{
  SET_TEXT( dialog_handle, ID_LAY_NEWFRAME  , MSG_LAYDEFAULT   );
  SET_TEXT( dialog_handle, ID_LAY_NEWLAT    , MSG_LAYLATIN     );
  SET_TEXT( dialog_handle, ID_LAY_NEWNAT    , MSG_LAYNATIONAL  );
  SET_TEXT( dialog_handle, ID_LAY_NEWSYS    , MSG_LAYSYSTEM    );
  SET_TEXT( dialog_handle, ID_LAY_CHNGFRAME , MSG_LAYCHANGER   );
  SET_TEXT( dialog_handle, ID_LAY_CHNGENABLE, MSG_CHNGENABLE   );
  SET_TEXT( dialog_handle, ID_LAY_CHNGSETKEY, MSG_ASSIGN       );

  ULONG default_layer  = ini_get_long( INI_LAYOUT_NEW, KL_NONE );
  BOOL  enable_changer = ini_get_long( INI_CHNGENABLE, TRUE    );

  if( default_layer == KL_NONE )
      default_layer = 2;

  memset( &changer_hotkeys, 0, sizeof(changer_hotkeys));
  changer_hotkeys.fs = KC_ALT | KC_CTRL | KC_SCANCODE;
  changer_hotkeys.scancode = 0x26; // L

  ini_get_data( INI_CHNGHOTKEYS, &changer_hotkeys, sizeof(changer_hotkeys));

  SET_TEXT    ( dialog_handle, ID_LAY_CHNGKEYS  , hotkey_string( &changer_hotkeys ));
  SET_CHECKBOX( dialog_handle, ID_LAY_CHNGENABLE, enable_changer );
  SET_RADIOBOX( dialog_handle, ID_LAY_NEWLAT+default_layer, TRUE );

  ENABLEID( dialog_handle, ID_LAY_CHNGSETKEY, enable_changer );
  ENABLEID( dialog_handle, ID_LAY_CHNGKEYS  , enable_changer );
}

/*--------------------------------------------------
 * Set layouts values to default
 *--------------------------------------------------*/
static void layouts_default( HWND dialog_handle )
{
  SET_CHECKBOX( dialog_handle, ID_LAY_NEWSYS    , TRUE );
  SET_CHECKBOX( dialog_handle, ID_LAY_CHNGENABLE, TRUE );

  memset( &changer_hotkeys, 0, sizeof(changer_hotkeys));
  changer_hotkeys.fs = KC_ALT | KC_CTRL | KC_SCANCODE;
  changer_hotkeys.scancode = 0x26;

  SET_TEXT( dialog_handle, ID_LAY_CHNGKEYS,
                           hotkey_string( &changer_hotkeys ));

  ENABLEID( dialog_handle, ID_LAY_CHNGSETKEY, TRUE );
  ENABLEID( dialog_handle, ID_LAY_CHNGKEYS  , TRUE );
}

/*--------------------------------------------------
 * Save layouts preferences
 *--------------------------------------------------*/
static void layouts_save()
{
  ULONG default_layer = GET_RADIOBOX( handle_of_layouts, ID_LAY_NEWSYS );

  if( default_layer != KL_LATIN && default_layer != KL_NATIONAL )
      default_layer  = KL_NONE;

  ini_put_long( INI_LAYOUT_NEW , default_layer );
  ini_put_data( INI_CHNGHOTKEYS, &changer_hotkeys, sizeof(changer_hotkeys));
  ini_put_long( INI_CHNGENABLE , GET_CHECKBOX( handle_of_layouts, ID_LAY_CHNGENABLE ));
}

/*--------------------------------------------------
 * Apply layouts preferences
 *--------------------------------------------------*/
static void layouts_apply()
{
  set_default_layer_id( ini_get_long( INI_LAYOUT_NEW, KL_NONE ));
  set_changer         ( ini_get_long( INI_CHNGENABLE, TRUE    ));

  memset( &changer_hotkeys, 0, sizeof(changer_hotkeys));
  changer_hotkeys.fs = KC_ALT | KC_CTRL | KC_SCANCODE;
  changer_hotkeys.scancode = 0x26;

  ini_get_data( INI_CHNGHOTKEYS, &changer_hotkeys, sizeof(changer_hotkeys));
  set_changer_hotkeys( &changer_hotkeys );
}

/*---------------------------------------------------
 * Layouts dialog message processing
 *--------------------------------------------------*/
MRESULT EXPENTRY layouts_dispatch( HWND hWnd, ULONG msg, MPARAM mp1, MPARAM mp2 )
{
  switch ( msg )
  {
    case WM_INITDLG:
      layouts_create   ( hWnd );
      return MRFROMLONG( TRUE );

    case WM_SETDEFAULT:
      layouts_default  ( hWnd );
      return MRFROMLONG( TRUE );

    case WM_COMMAND:
      switch( SHORT1FROMMP(mp1))
      {
        case ID_LAY_CHNGSETKEY:
        {
          BOOL changer_enable = is_changer_enabled();
          set_changer( FALSE );

          hotkey_assign( hWnd, &changer_hotkeys );
          SET_TEXT( hWnd, ID_LAY_CHNGKEYS, hotkey_string( &changer_hotkeys ));

          set_changer( changer_enable );
          return MRFROMLONG ( TRUE );
        }

        case DID_CANCEL:
          return MRFROMLONG( TRUE );
      }
      break;

    case WM_CONTROL:
      if( SHORT2FROMMP(mp1) == CBN_EFCHANGE && SHORT1FROMMP(mp1) == ID_LAY_CHNGENABLE )
      {
        BOOL enable_changer = GET_CHECKBOX( hWnd, ID_LAY_CHNGENABLE );

        ENABLEID( hWnd, ID_LAY_CHNGSETKEY, enable_changer );
        ENABLEID( hWnd, ID_LAY_CHNGKEYS  , enable_changer );
      }
      break;
  }
  return WinDefDlgProc( hWnd, msg, mp1, mp2 );
}

/*---------------------------------------------------
 * Create language dialog
 *--------------------------------------------------*/
static void language_create( HWND dialog_handle )
{
  SET_TEXT( dialog_handle, ID_LNG_FRAME , MSG_LNGSELECT  );
  ADD_ITEM( dialog_handle, ID_LNG_SELECT, MSG_LNGAUTO    );
  ADD_ITEM( dialog_handle, ID_LNG_SELECT, MSG_LNGRUSSIAN );
  ADD_ITEM( dialog_handle, ID_LNG_SELECT, MSG_LNGENGLISH );
  SEL_ITEM( dialog_handle, ID_LNG_SELECT, ini_get_short( INI_LANGUAGE, 0 ));
}

/*--------------------------------------------------
 * Set language values to default
 *--------------------------------------------------*/
static void language_default( HWND dialog_handle )
{
  SEL_ITEM( dialog_handle, ID_LNG_SELECT, 0 );
}

/*--------------------------------------------------
 * Save language preferences
 *--------------------------------------------------*/
static void language_save()
{
  ini_put_short( INI_LANGUAGE, GET_ITEM( handle_of_language, ID_LNG_SELECT ));
}

/*--------------------------------------------------
 * Apply language preferences
 *--------------------------------------------------*/
static void language_apply()
{
  SHORT country;
  SHORT language = ini_get_short( INI_LANGUAGE, 0 );

  switch( language )
  {
    case  1: country = 7; break;
    case  2: country = 1; break;
    default: country = 0;
  }

  KLMsg::select_country( country );
}

/*--------------------------------------------------
 * Language dialog message processing
 *--------------------------------------------------*/
MRESULT EXPENTRY language_dispatch( HWND hWnd, ULONG msg, MPARAM mp1, MPARAM mp2 )
{
  switch ( msg )
  {
    case WM_INITDLG:
      language_create( hWnd );
      break;

    case WM_SETDEFAULT:
      language_default ( hWnd );
      return MRFROMLONG( TRUE );

    case WM_COMMAND:
      if( SHORT1FROMMP(mp1) == DID_CANCEL )
        return MRFROMLONG( TRUE );

      break;
  }
  return WinDefDlgProc( hWnd, msg, mp1, mp2 );
}

/*--------------------------------------------------
 * Ceate About dialog
 *--------------------------------------------------*/
static void about_create( HWND dialog_handle )
{
  char ver[64];
  sprintf( ver, "Version %s", PACKAGE_VERSION );
  SET_TEXT( dialog_handle, ID_ABO_VERSION, ver );
}

/*--------------------------------------------------
 * About dialog message processing
 *--------------------------------------------------*/
MRESULT EXPENTRY about_dispatch( HWND hWnd, ULONG msg, MPARAM mp1, MPARAM mp2 )
{
  switch ( msg )
  {
    case WM_INITDLG:
      about_create( hWnd );
      break;

    case WM_COMMAND:
      if( SHORT1FROMMP(mp1) == DID_CANCEL )
        return MRFROMLONG(TRUE);

      break;
  }
  return WinDefDlgProc( hWnd, msg, mp1, mp2 );
}

/*--------------------------------------------------
 * Create and initialize notebook
 *--------------------------------------------------*/
static void notebook_create( HWND dialog_handle  )
{
  MRESULT id;
  char status[256];

  handle_of_notebook = WinWindowFromID( dialog_handle, ID_NBK_SETUP );

  // Setup notebook

  WinSendMsg( handle_of_notebook, BKM_SETDIMENSIONS,
              MPFROM2SHORT(100,25), MPFROMSHORT(BKA_MAJORTAB));

  WinSendMsg( handle_of_notebook, BKM_SETDIMENSIONS,
              MPFROMLONG (0), MPFROMSHORT(BKA_MINORTAB));

  WinSendMsg( handle_of_notebook, BKM_SETNOTEBOOKCOLORS,
              MPFROMLONG (SYSCLR_FIELDBACKGROUND),
              MPFROMSHORT(BKA_BACKGROUNDPAGECOLORINDEX));

  // Load of dialog templates

  handle_of_setup    = WinLoadDlg( handle_of_notebook,
                                   handle_of_notebook,
                                   setup_dispatch, 0, ID_WIN_SETUP, 0 );

  handle_of_language = WinLoadDlg( handle_of_notebook,
                                   handle_of_notebook,
                                   language_dispatch, 0, ID_WIN_LANGUAGE, 0 );

  handle_of_layouts  = WinLoadDlg( handle_of_notebook,
                                   handle_of_notebook,
                                   layouts_dispatch, 0, ID_WIN_LAYOUTS, 0 );

  handle_of_about    = WinLoadDlg( handle_of_notebook,
                                   handle_of_notebook,
                                   about_dispatch, 0, ID_WIN_ABOUT, 0 );

  // Add setup page 1

  sprintf( status, MSG_PAGEOF, 1, 2 );

  id = WinSendMsg( handle_of_notebook, BKM_INSERTPAGE,
                   0,
                   MPFROM2SHORT( BKA_AUTOPAGESIZE |
                                 BKA_MAJOR        |
                                 BKA_STATUSTEXTON , BKA_LAST ));

  WinSendMsg( handle_of_notebook, BKM_SETTABTEXT,
              MPFROMLONG(id), MPFROMP(MSG_SETUP));
  WinSendMsg( handle_of_notebook, BKM_SETSTATUSLINETEXT,
              MPFROMLONG(id), MPFROMP(status));
  WinSendMsg( handle_of_notebook, BKM_SETPAGEWINDOWHWND,
              MPFROMLONG(id), MPFROMHWND(handle_of_setup));

  // Add setup page 2 (layouts page)

  sprintf( status, MSG_PAGEOF, 2, 2 );

  id = WinSendMsg( handle_of_notebook, BKM_INSERTPAGE,
                   0,
                   MPFROM2SHORT( BKA_AUTOPAGESIZE |
                                 BKA_MINOR        |
                                 BKA_STATUSTEXTON , BKA_LAST));

  WinSendMsg( handle_of_notebook, BKM_SETTABTEXT,
              MPFROMLONG(id), MPFROMP(status));
  WinSendMsg( handle_of_notebook, BKM_SETSTATUSLINETEXT,
              MPFROMLONG(id), MPFROMP(status));
  WinSendMsg( handle_of_notebook, BKM_SETPAGEWINDOWHWND,
              MPFROMLONG(id), MPFROMHWND(handle_of_layouts));

  // Add language page

  id = WinSendMsg( handle_of_notebook, BKM_INSERTPAGE,
                   0,
                   MPFROM2SHORT( BKA_AUTOPAGESIZE |
                                 BKA_MAJOR        , BKA_LAST));

  WinSendMsg( handle_of_notebook, BKM_SETTABTEXT,
              MPFROMLONG(id), MPFROMP(MSG_LANGUAGE));
  WinSendMsg( handle_of_notebook, BKM_SETPAGEWINDOWHWND,
              MPFROMLONG(id), MPFROMHWND(handle_of_language));

  // Add about page

  id = WinSendMsg( handle_of_notebook, BKM_INSERTPAGE,
                   0,
                   MPFROM2SHORT( BKA_AUTOPAGESIZE |
                                 BKA_MAJOR        , BKA_LAST));

  WinSendMsg( handle_of_notebook, BKM_SETTABTEXT,
              MPFROMLONG(id), MPFROMP(MSG_ABOUT   ));
  WinSendMsg( handle_of_notebook, BKM_SETPAGEWINDOWHWND,
              MPFROMLONG(id), MPFROMHWND(handle_of_about));

  SET_TEXT( dialog_handle, DID_OK        , MSG_OK      );
  SET_TEXT( dialog_handle, ID_SET_DEFAULT, MSG_DEFAULT );
  SET_TEXT( dialog_handle, DID_CANCEL    , MSG_CANCEL  );
}

/*--------------------------------------------------
 * Destroy notebook
 *--------------------------------------------------*/
static void notebook_destroy()
{
  if( handle_of_setup    ) WinDestroyWindow( handle_of_setup    );
  if( handle_of_language ) WinDestroyWindow( handle_of_language );
  if( handle_of_layouts  ) WinDestroyWindow( handle_of_layouts  );
  if( handle_of_about    ) WinDestroyWindow( handle_of_about    );

  handle_of_setup    = 0;
  handle_of_language = 0;
  handle_of_layouts  = 0;
  handle_of_about    = 0;
}

/*--------------------------------------------------
 * Save preferences
 *--------------------------------------------------*/
static void notebook_save()
{
  setup_save        ();
  layouts_save      ();
  language_save     ();
  apply_preferences ();
}

/*--------------------------------------------------
 * Notebook messages processing
 *--------------------------------------------------*/
MRESULT EXPENTRY notebook_dispatch( HWND hWnd, ULONG msg, MPARAM mp1, MPARAM mp2 )
{
  switch ( msg )
  {
    case WM_INITDLG:
      notebook_create( hWnd );
      break;

    case WM_DESTROY:
      notebook_destroy();
      break;

    case WM_COMMAND:
      switch( SHORT1FROMMP(mp1))
      {
        case DID_OK:
          notebook_save();
          break;


        case ID_SET_DEFAULT:
        {
          LONG id;
          HWND hwnd;

          id = (LONG)WinSendMsg( handle_of_notebook, BKM_QUERYPAGEID, 0,
                                 MPFROM2SHORT(BKA_TOP,BKA_MAJOR));

          if( id && id != BOOKERR_INVALID_PARAMETERS )
              hwnd = (HWND)WinSendMsg( handle_of_notebook,
                                       BKM_QUERYPAGEWINDOWHWND, MPFROMLONG(id), 0 );

          if( hwnd && hwnd != BOOKERR_INVALID_PARAMETERS )
            WinSendMsg( hwnd, WM_SETDEFAULT, 0, 0 );

          return MRFROMLONG(1L);
        }
      }
      break;
  }
  return WinDefDlgProc( hWnd, msg, mp1, mp2 );
}

/*--------------------------------------------------
 * Edit user preferences
 *--------------------------------------------------*/
void setup_preferences()
{
  WinDlgBox( HWND_DESKTOP, handle_of_indicator, notebook_dispatch,
                           NULLHANDLE, ID_WIN_NOTEBOOK, NULL );
}

/*--------------------------------------------------
 * Apply preferences
 *--------------------------------------------------*/
void apply_preferences()
{
  setup_apply   ();
  layouts_apply ();
  language_apply();

  WinSendMsg( handle_of_indicator, WM_LAYERCHANGED, 0, 0 );
}

/*--------------------------------------------------
 * Dispatch messages
 *--------------------------------------------------*/
MRESULT EXPENTRY hotkey_dispatch( HWND hWnd, ULONG msg, MPARAM mp1, MPARAM mp2 )
{
  static CHRMSG* keys;

  switch ( msg )
  {
    case WM_INITDLG:
      SET_TEXT( hWnd, ID_KEY_TEXT, MSG_KEYPROMPT );
      keys = (CHRMSG*)mp2;
      break;

    case WM_CHAR:
      if( !( CHARMSG(&msg)->fs & KC_KEYUP )
          && CHARMSG(&msg)->vkey != VK_NUMLOCK
          && CHARMSG(&msg)->vkey != VK_ESC
          && CHARMSG(&msg)->vkey != VK_PRINTSCRN
          && CHARMSG(&msg)->vkey != VK_PAUSE
          && CHARMSG(&msg)->vkey != VK_SHIFT
          && CHARMSG(&msg)->vkey != VK_CTRL
          && CHARMSG(&msg)->vkey != VK_ALT
          && CHARMSG(&msg)->vkey != VK_CAPSLOCK
          && CHARMSG(&msg)->vkey != VK_SCRLLOCK
          && CHARMSG(&msg)->scancode
        )
      {
        *keys = *CHARMSG(&msg);
        WinPostMsg( hWnd, WM_CLOSE, 0 , 0 );
        return MRFROMLONG(TRUE);
      }
      break;
  }
  return WinDefDlgProc( hWnd, msg, mp1, mp2 );
}

/*--------------------------------------------------
 * Get keyboard combination
 *--------------------------------------------------*/
void hotkey_assign( HWND owner, CHRMSG* keys )
{
  WinDlgBox( HWND_DESKTOP, owner, hotkey_dispatch,
                           NULLHANDLE, ID_WIN_KEYPROMPT, keys );
}

/*--------------------------------------------------
 * Convert keyboard combination to string
 *--------------------------------------------------*/
const char* hotkey_string( const CHRMSG* keys )
{
  static char  buffer[125];
  static char* key;


  buffer[0] = 0;

  if( keys->fs & KC_CTRL  ) strcat( buffer, "<Ctrl>" );
  if( keys->fs & KC_ALT   ) strcat( buffer, "<Alt>"  );
  if( keys->fs & KC_SHIFT ) strcat( buffer, "<Shft>" );

  switch( keys->scancode )
  {
    case 0x01: key = "<Esc>"    ; break;
    case 0x02: key = "1"        ; break;
    case 0x03: key = "2"        ; break;
    case 0x04: key = "3"        ; break;
    case 0x05: key = "4"        ; break;
    case 0x06: key = "5"        ; break;
    case 0x07: key = "6"        ; break;
    case 0x08: key = "7"        ; break;
    case 0x09: key = "8"        ; break;
    case 0x0A: key = "9"        ; break;
    case 0x0B: key = "0"        ; break;
    case 0x0C: key = "-"        ; break;
    case 0x0D: key = "="        ; break;
    case 0x0E: key = "<Bksp>"   ; break;
    case 0x0F: key = "<Tab>"    ; break;
    case 0x10: key = "Q"        ; break;
    case 0x11: key = "W"        ; break;
    case 0x12: key = "E"        ; break;
    case 0x13: key = "R"        ; break;
    case 0x14: key = "T"        ; break;
    case 0x15: key = "Y"        ; break;
    case 0x16: key = "U"        ; break;
    case 0x17: key = "I"        ; break;
    case 0x18: key = "O"        ; break;
    case 0x19: key = "P"        ; break;
    case 0x1A: key = "["        ; break;
    case 0x1B: key = "]"        ; break;
    case 0x1C: key = "<Enter>"  ; break;
    case 0x1D: key = "<Ctrl>"   ; break;
    case 0x1E: key = "A"        ; break;
    case 0x1F: key = "S"        ; break;
    case 0x20: key = "D"        ; break;
    case 0x21: key = "F"        ; break;
    case 0x22: key = "G"        ; break;
    case 0x23: key = "H"        ; break;
    case 0x24: key = "J"        ; break;
    case 0x25: key = "K"        ; break;
    case 0x26: key = "L"        ; break;
    case 0x27: key = ";"        ; break;
    case 0x28: key = "\""       ; break;
    case 0x29: key = "`"        ; break;
    case 0x2A: key = "<LShft>"  ; break;
    case 0x2B: key = "\\"       ; break;
    case 0x2C: key = "Z"        ; break;
    case 0x2D: key = "X"        ; break;
    case 0x2E: key = "C"        ; break;
    case 0x2F: key = "V"        ; break;
    case 0x30: key = "B"        ; break;
    case 0x31: key = "N"        ; break;
    case 0x32: key = "M"        ; break;
    case 0x33: key = ","        ; break;
    case 0x34: key = "."        ; break;
    case 0x35: key = "/"        ; break;
    case 0x36: key = "<RShft>"  ; break;
    case 0x37: key = "<PrtSc>"  ; break;
    case 0x38: key = "<Alt>"    ; break;
    case 0x39: key = "<Space>"  ; break;
    case 0x3A: key = "<Caps>"   ; break;
    case 0x3B: key = "<F1>"     ; break;
    case 0x3C: key = "<F2>"     ; break;
    case 0x3D: key = "<F3>"     ; break;
    case 0x3E: key = "<F4>"     ; break;
    case 0x3F: key = "<F5>"     ; break;
    case 0x40: key = "<F6>"     ; break;
    case 0x41: key = "<F7>"     ; break;
    case 0x42: key = "<F8>"     ; break;
    case 0x43: key = "<F9>"     ; break;
    case 0x44: key = "<F10>"    ; break;
    case 0x45: key = "<Num>"    ; break;
    case 0x46: key = "<Scrl>"   ; break;
    case 0x47: key = "<Home>"   ; break;
    case 0x48: key = "<Up>"     ; break;
    case 0x49: key = "<PgUp>"   ; break;
    case 0x4A: key = "<->"      ; break;
    case 0x4B: key = "<Left>"   ; break;
    case 0x4C: key = "<>"       ; break;
    case 0x4D: key = "<Right>"  ; break;
    case 0x4E: key = "<+>"      ; break;
    case 0x4F: key = "<End>"    ; break;
    case 0x50: key = "<Down>"   ; break;
    case 0x51: key = "<PgDn>"   ; break;
    case 0x52: key = "<Ins>"    ; break;
    case 0x53: key = "<Del>"    ; break;
    case 0x57: key = "<F11>"    ; break;
    case 0x58: key = "<F12>"    ; break;
    case 0x5A: key = "<*Enter>" ; break;
    case 0x5C: key = "</>"      ; break;

    case 0x60: key = "<*Home>"  ; break;
    case 0x61: key = "<*Up>"    ; break;
    case 0x62: key = "<*PgUp>"  ; break;
    case 0x63: key = "<*Left>"  ; break;
    case 0x64: key = "<*Right>" ; break;
    case 0x65: key = "<*End>"   ; break;
    case 0x66: key = "<*Down>"  ; break;
    case 0x67: key = "<*PgDn>"  ; break;
    case 0x68: key = "<*Ins>"   ; break;
    case 0x69: key = "<*Del>"   ; break;

    case 0x7C: key = "<Menu>"   ; break;
    case 0x7E: key = "<LWin>"   ; break;
    case 0x7F: key = "<RWin>"   ; break;

    default:
      key = 0;
  }

  if( !key )
  {
    strcat( buffer, "0x" );
    ultoa ( keys->scancode, buffer + strlen(buffer), 16 );
  }
  else
    strcat( buffer, key  );

  return buffer;
}

