/* Keyboard Layer/2
 *
 * Virtual Device Driver
 * Copyright (C) 1998 by Dmitry A.Steklenev
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Dmitry A.Steklenev".
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR OR CONTRIBUTORS "AS IS"
 * AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * AUTHOR OR THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: layer.c,v 1.2 2002/03/31 19:08:54 glass Exp $
 */

#define  INCL_VDDHELPERS
#include <mvdm.h>

#include "layer.h"
#include "layer_global.h"
#include "layer_hotkey.h"

#pragma data_seg()
/*--------------------------------------------------
 * Global data
 *--------------------------------------------------*/

BOOL    enable_beeps     = TRUE;
BOOL    enable_vdd       = FALSE;
HVDHSEM data_ready       = 0;
ULONG   default_layer_id = KL_NONE;
BYTE    sec_border       = 0x0C;
BOOL    is_extended_key  = FALSE;

struct  VLEVENT Event;

#pragma data_seg( MVDMINSTDATA )
/*--------------------------------------------------
 * VDM instanse data
 *--------------------------------------------------*/

BOOL  is_border_painted;
BYTE  pri_border;
SHORT last_state;
BYTE  last_scan;
HHOOK hHookInt10Ret;

#pragma alloc_text( CSWAP_TEXT, VLBeepLat )
/*--------------------------------------------------
 * Sound signal at turn on of the latin layout
 *--------------------------------------------------*/
void PRIVENTRY VLBeepLat(void)
{
  if( enable_beeps )
      VDHDevBeep( BEEP_LAT, BEEP_LEN );
}

#pragma alloc_text( CSWAP_TEXT, VLBeepNat )
/*--------------------------------------------------
 * Sound signal at turn on of the national layout
 *--------------------------------------------------*/
void PRIVENTRY VLBeepNat(void)
{
  if( enable_beeps )
      VDHDevBeep( BEEP_NAT, BEEP_LEN );
}

#pragma alloc_text( CSWAP_TEXT, get_layer_id )
/*--------------------------------------------------
 * Returns layer for current VDM
 *--------------------------------------------------*/
ULONG PRIVENTRY get_layer_id()
{
  return ROMBios.rb_fbKFlag2 & BIOSKFLAG2_CYR ? KL_NATIONAL : KL_LATIN;
}

#pragma alloc_text( CSWAP_TEXT, set_layer_id )
/*--------------------------------------------------
 * Sets layer for current VDM
 *--------------------------------------------------*/
void PRIVENTRY set_layer_id( ULONG id )
{
  if( id == KL_LATIN    )
      ROMBios.rb_fbKFlag2 &= ~BIOSKFLAG2_CYR;
  if( id == KL_NATIONAL )
      ROMBios.rb_fbKFlag2 |=  BIOSKFLAG2_CYR;
}

#pragma alloc_text( CSWAP_TEXT, VLPostEvent )
/*--------------------------------------------------
 * Send message about layer status changes
 *--------------------------------------------------*/
void PRIVENTRY VLPostEvent(void)
{
  Event.EventType = VLEVN_LAYERCHANGED;
  Event.EventData = get_layer_id();
  Event.EventPID  = VDHQuerySysValue( 0, VDHLSV_PID );
  Event.EventNULL = 0;

  VDHPostEventSem(data_ready);
}

#pragma alloc_text( CSWAP_TEXT, VLInt10Return )
/*--------------------------------------------------
 * Return from INT 10
 *--------------------------------------------------*/
VOID HOOKENTRY VLInt10Return( PVOID p, PCRF pcrf )
{
  is_border_painted = FALSE;
  VDHPopRegs( VDHREG_AX | VDHREG_BX );
}

#pragma alloc_text( CSWAP_TEXT, VLSetBorder )
/*--------------------------------------------------
 * Set screen border color
 *--------------------------------------------------*/
void PRIVENTRY VLSetBorder( PCRF pcrf )
{
  BYTE color = pri_border;

  is_border_painted = TRUE;
  VDHPushRegs( VDHREG_AX | VDHREG_BX );

  if( get_layer_id() == KL_NATIONAL )
      color = sec_border;

  // ĴINT 10H 0bH: Select Graphics Palette or Text Border Color
  //
  // Expects: AH    0bH
  //          BL    text modes: select overscan (border) color
  //                     00H-0fH low-intensity colors
  //                     10H-1fH high-intensity colors
  //                graphics:   select palette combination
  //                     0 = green/red/brown
  //                     1 = cyan/magenta/white
  //
  // ĴINT 10H 1001H: Set Overscan/Border Color Register
  //
  // Expects: AX    1001H
  //          BH    6-bit RGB color to display for that attribute
  //
  //  Info: This sets the EGA/VGA screen border to a specific rgbRGB color.
  //
  //  BH is a 6-bit rgbRGB color setting, as follows:
  //
  //        76543210
  //           rgbRGB             Note: R,G,B are 2/3-intensity
  //        ҽ bit               r,g,b are 1/3-intensity
  //                   0: 01H Blue
  //                  1: 02H Green
  //                 2: 04H Red
  //                3: 08H blue'
  //               4: 10H green' (also CGA-emulation intensity)
  //              5: 20H red'

  if( color & 0x80 )
  {
    AH(pcrf) = 0x0B;
    BH(pcrf) = 0x00;
    BL(pcrf) = color & 0x7F;
  }
  else
  {
    AX(pcrf) = 0x1001;
    BH(pcrf) = color;
  }

  VDHPushInt( 0x10 );
  VDHArmReturnHook( hHookInt10Ret, VDHARH_NORMAL_IRET );
}

#pragma alloc_text( CSWAP_TEXT, VLInt10Proc )
/*--------------------------------------------------
 * INT 10 Handler
 *--------------------------------------------------*/
BOOL VDHENTRY VLInt10Proc( PCRF pcrf )
{
  // ĴINT 10H 0bH: Select Graphics Palette or Text Border Color
  //
  // Expects: AH    0bH
  //          BL    text modes: select overscan (border) color
  //                     00H-0fH low-intensity colors
  //                     10H-1fH high-intensity colors
  //                graphics:   select palette combination
  //                     0 = green/red/brown
  //                     1 = cyan/magenta/white
  //
  // ĴINT 10H 1001H: Set Overscan/Border Color Register
  //
  // Expects: AX    1001H
  //          BH    6-bit RGB color to display for that attribute
  //
  //  Info: This sets the EGA/VGA screen border to a specific rgbRGB color.
  //
  //  BH is a 6-bit rgbRGB color setting, as follows:
  //
  //        76543210
  //           rgbRGB             Note: R,G,B are 2/3-intensity
  //        ҽ bit               r,g,b are 1/3-intensity
  //                   0: 01H Blue
  //                  1: 02H Green
  //                 2: 04H Red
  //                3: 08H blue'
  //               4: 10H green' (also CGA-emulation intensity)
  //              5: 20H red'

  if( !is_border_painted && enable_vdd )
  {
     if( AX(pcrf) == 0x1001 )
       pri_border = BH(pcrf) & 0x3F;

     if( AH(pcrf) == 0x0B && BH(pcrf) == 0 )
       pri_border = BL(pcrf) | 0x80;
  }
  return FALSE;
}

#pragma alloc_text( CSWAP_TEXT, VLInt15Proc )
/*--------------------------------------------------
 * INT 15 Handler
 *--------------------------------------------------*/
BOOL VDHENTRY VLInt15Proc( PCRF pcrf )
{
  BYTE scan;
  LONG detected;

  // ĴINT 15H 4fH: Keyboard Intercept
  //
  // Expects: AH    4fH
  //          AL    keyboard scan code received from hardware
  //          CX    process ID
  //          
  // Returns: AL    keyboard scan code
  //          CF = CY: replace input scan code with the one on AL
  //          CF = NC: no change; use hardware scan code
  // 

  if( !enable_vdd || AH(pcrf) != 0x4F )
    return FALSE;

  // To check up, if the status of the layer is changed by anyone,
  // will notify PM and to establish the appropriate colour of a
  // screen border:

  if( last_state != get_layer_id())
  {
    last_state = get_layer_id();
    VLPostEvent( );
    VLSetBorder(0);
  }

  if( hkey_selected() == XS_DONTME )
    return FALSE;

  scan = AL(pcrf);

  if( scan == 0xE0 )       // Extended scan code
  {
    is_extended_key = TRUE;
    return FALSE;
  }

  if( is_extended_key )   // Convert extended key to PM scan code
  {
    switch( scan & 0x7F )
    {
      case CS_LALT     : scan = CS_RALT;     break;
      case CS_LCONTROL : scan = CS_RCONTROL; break;
      case 0x5B        : scan = CS_LWIN;     break;
      case 0x5C        : scan = CS_RWIN;     break;
      case 0x5D        : scan = CS_WINMENU;  break;
    }

    scan |= ( AL(pcrf) & 0x80 );
    is_extended_key = FALSE;
  }

  if( scan == last_scan )  // If it is auto repeat key, did not
    return FALSE;          // interested :)

  last_scan = scan;
  detected  = hkey_dispatch( scan, get_layer_id(), ROMBios.rb_Ticks );

  if( detected != KL_NONE )
  {
    set_layer_id( detected );
    last_state = get_layer_id();

    if( detected == KL_LATIN    )
      VLBeepLat();
    else
      VLBeepNat();

    VLPostEvent();
    VLSetBorder(pcrf);
  }

  return FALSE;
}

#pragma alloc_text( CSWAP_TEXT, VLCreateVDM )
/*--------------------------------------------------
 * Create VDM
 *--------------------------------------------------*/
BOOL HOOKENTRY VLCreateVDM( HVDM hvdm )
{
  is_border_painted = FALSE;
  pri_border        = 0;
  last_state        = 0;

  VDHInstallIntHook( hvdm, 0x15, VLInt15Proc, !VDH_ASM_HOOK );
  VDHInstallIntHook( hvdm, 0x10, VLInt10Proc, !VDH_ASM_HOOK );

  hHookInt10Ret = VDHAllocHook( VDH_RETURN_HOOK, (PFNARM)VLInt10Return, 0 );

  if( !hHookInt10Ret )
    return FALSE;

  if( enable_vdd && default_layer_id != KL_NONE )
    set_layer_id( default_layer_id );

  return TRUE;
}

#pragma alloc_text( CSWAP_TEXT, VLCodePageChange )
/*--------------------------------------------------
 * CodePage Change
 *--------------------------------------------------*/
BOOL HOOKENTRY VLCodePageChange( ULONG codepage )
{
  if( enable_vdd && default_layer_id != KL_NONE )
    set_layer_id( default_layer_id );

  VLPostEvent();
  return TRUE;
}

#pragma alloc_text( CSWAP_TEXT, VLSysReqProc )
/*--------------------------------------------------
 * VDD Interface
 *--------------------------------------------------*/
LONG EXPENTRY VLSysReqProc( SGID  sgid,
                            ULONG ulFunc,
                            ULONG nbIn,
                            PVOID pIn,
                            ULONG nbOut,
                            PVOID pOut    )
{
  switch( ulFunc )
  {
    // Sets switch sequence
    case VLCMD_SETSWITCH:
    {
      if( nbIn == sizeof(SHORT))
        hkey_select( *(SHORT*)pIn );

      break;
    }

    // Enables/disables sound
    case VLCMD_SETBEEP:

      if( nbIn == sizeof(BOOL))
          enable_beeps = *(BOOL*)pIn;

      break;


    // Sets VDM screen border color
    case VLCMD_SETCOLOR:
    {
      if( nbIn == sizeof(LONG))
      {
        RGB  color  = *(RGB*)pIn;
        BYTE border = 0;

        switch(( color.bBlue  + 50 ) / 85 )
        {
          case  0 : break;
          case  1 : border |= 0x08; break;
          default : border |= 0x01; break;
        }

        switch(( color.bGreen + 50 ) / 85 )
        {
          case  0 : break;
          case  1 : border |= 0x10; break;
          default : border |= 0x02;
        }

        switch(( color.bRed   + 50 ) / 85 )
        {
          case  0 : break;
          case  1 : border |= 0x20; break;
          default : border |= 0x04;
        }

        sec_border = border;
      }
      break;
    }

    // Sets time-out (witch convert from tens to ticks)
    case VLCMD_SETTMOUT:
    {
      if( nbIn >= sizeof(ULONG))
          hkey_set_timeout( (*(ULONG*)pIn * 100 )/55 );

      break;
    }

    // Gets layer status
    case VLCMD_GETLAYER:
    {
      if( nbOut == sizeof(SHORT))
      {
        HVDM hvdm = VDHHandleFromPID(sgid);

        if( hvdm )
          (*(SHORT*)pOut) = pROMBios(hvdm)->rb_fbKFlag2 & BIOSKFLAG2_CYR ? 1 : 0;
        else
          (*(SHORT*)pOut) = 0;
      }
      break;
    }

    // Gets VDD event
    case VLCMD_GETEVENT:
    {
      if( nbOut == sizeof(struct VLEVENT))
      {
        VDHWaitEventSem ( data_ready, SEM_INDEFINITE_WAIT );
        VDHResetEventSem( data_ready );
        (*(struct VLEVENT*)pOut) = Event;
      }
      break;
    }


    // Enables/disables VDM Support
    case VLCMD_ENABLE:
    {
      if( nbIn == sizeof(BOOL))
          enable_vdd = *(BOOL*)pIn;

      break;
    }


    // Sets layout for new windows
    case VLCMD_SETFORNEW:
    {
      if( nbIn == sizeof(ULONG))
          default_layer_id = *(ULONG*)pIn;

      break;
    }

    // Unknown command
    default:
      return 1;
  }

  return 0;
}

#pragma entry(VLInit)
#pragma alloc_text( CINIT_TEXT, VLInit )
/*--------------------------------------------------
 * VDD Initialization
 *--------------------------------------------------*/
BOOL EXPENTRY VLInit( PSZ psz )
{
  VDHInstallUserHook( VDM_CREATE, VLCreateVDM  );
  VDHInstallUserHook( VDM_CODEPAGE_CHANGE, VLCodePageChange );

  if( !VDHCreateSem( &data_ready, VDH_EVENTSEM ))
    return FALSE;

  VDHResetEventSem( data_ready );
  Event.EventType = 0;
  Event.EventData = 0;
  Event.EventPID  = 0;
  Event.EventNULL = 0;

  VDHRegisterVDD( "LAYER$", VLSysReqProc, NULL );
  return TRUE;
}

